#include <Riostream.h>

extern TRandom *gRandom;
extern TBenchmark *gBenchmark;
extern TSystem *gSystem;

void ReadCFHeavyFlavourOutput() {

	// example macro to read the CF produced for D->Kpi
	// 8 steps considered
	// 12 variables
	
	gROOT->SetStyle("Plain");
	gStyle->SetPalette(1);
	gStyle->SetOptStat(0);
	gStyle->SetPalette(1);
	gStyle->SetCanvasColor(0);
	gStyle->SetFrameFillColor(0);
	gStyle->SetOptTitle(0);
	
	gSystem->SetIncludePath("-I. -I$ALICE_ROOT/include  -I$ROOTSYS/include");
	gSystem->Load("libANALYSIS.so");
	gSystem->Load("libANALYSISalice.so");
	gSystem->Load("$ALICE_ROOT/CORRFW/libCORRFW.so") ;
	
	// Setting up the container grid... 
	
	// Flag the sel steps 
	Int_t stepGen=0;
	Int_t stepGenAcc=1;
	Int_t stepGenVertex=2;
	Int_t stepGenRefit=3;
	Int_t stepRec=4;
	Int_t stepRecAcc=5;
	Int_t stepRecITSClus=6;
	Int_t stepRecPPR=7;
	
	// The sensitive variables, their indeces
	Int_t ipt =0;
	Int_t iy=1;
	UInt_t icTS  = 2;   //cTS stands for cosThetaStar
	UInt_t iptPi  = 3;
	UInt_t iptK  = 4;
	UInt_t icT  = 5;
	UInt_t idca  = 6;
	UInt_t id0pi  = 7;
	UInt_t id0K  = 8;
	UInt_t id0xd0  = 9;
	UInt_t ipointing  = 10;
	UInt_t iphi  = 11;
	
	// Get the container
	TFile* f = new TFile("AnalysisResults.root");	
	TDirectoryFile* d = (TDirectoryFile*)f->Get("PWG3_D2H_CFtaskD0toKpi");
	AliCFContainer *data = (AliCFContainer*) (d->Get("container"));
	
	// Make some 1 & 2-D projections.
	// MC level
	TCanvas *cmc1 =new TCanvas("cmc1","The MC distributions",0,0,1600,1200);  // canvas for all steps, 1st three variables
	TCanvas *cmc2 =new TCanvas("cmc2","The MC distributions",0,0,1600,1200);  // canvas for all steps, 2nd three variables 

	TCanvas *cmc3 =new TCanvas("cmc3","The MC distributions",0,0,1600,1200);  // canvas for all steps, 2nd three variables 
	TCanvas *cmc4 =new TCanvas("cmc4","The MC distributions",0,0,1600,765);  // canvas for all steps, 2nd three variables 
	TCanvas *cmcpt = new TCanvas("cmcpt","pt distribution from MC",50,50,550,550);
	TCanvas *cmcy = new TCanvas("cmcy","y distribution from MC",50,50,550,550);
	TCanvas *cmccTS = new TCanvas("cmcTS","cosThetaStar distribution from MC",50,50,550,550);  // cTS stands for cosThetaStar
	TCanvas *cmcptPi = new TCanvas("cmcptPi","pt_pi distribution from MC",50,50,550,550);
	TCanvas *cmcptK = new TCanvas("cmcptK","pt_K distribution from MC",50,50,550,550);
	TCanvas *cmccT = new TCanvas("cmccT","cT distribution from MC",50,50,550,550);
	TCanvas *cmcdca = new TCanvas("cmcdca","dca distribution from MC",50,50,550,550);
	TCanvas *cmcd0pi = new TCanvas("cmcd0pi","d0pi distribution from MC",50,50,550,550);
	TCanvas *cmcd0K = new TCanvas("cmcd0K","d0K distribution from MC",50,50,550,550);
	TCanvas *cmcd0xd0 = new TCanvas("cmcd0xd0","d0xd0 distribution from MC",50,50,550,550);
	TCanvas *cmcpointing = new TCanvas("cmcpointing","cosPointingAngle distribution from MC",50,50,550,550);
	TCanvas *cmcphi = new TCanvas("cmcphi","Phi distribution from MC",50,50,550,550);

	// MC Acceptance level
	TCanvas *cmcAccpt = new TCanvas("cmcAccpt","pt distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAccy = new TCanvas("cmcAccy","y distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAcccTS = new TCanvas("cmcAccTS","cosThetaStar distribution from MC - Acceptance cuts",50,50,550,550);  // cTS stands for cosThetaStar
	TCanvas *cmcAccptPi = new TCanvas("cmcAccptPi","pt_pi distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAccptK = new TCanvas("cmcAccptK","pt_K distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAcccT = new TCanvas("cmcAcccT","cT distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAccdca = new TCanvas("cmcAccdca","dca distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAccd0pi = new TCanvas("cmcAccd0pi","d0pi distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAccd0K = new TCanvas("cmcAccd0K","d0K distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAccd0xd0 = new TCanvas("cmcAccd0xd0","d0xd0 distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAccpointing = new TCanvas("cmcAccpointing","cosPointingAngle distribution from MC - Acceptance cuts",50,50,550,550);
	TCanvas *cmcAccphi = new TCanvas("cmcAccphi","Phi distribution from MC - Acceptance cuts",50,50,550,550);

	// MC Vertex level
	TCanvas *cmcVertexpt = new TCanvas("cmcVertexpt","pt distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexy = new TCanvas("cmcVertexy","y distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexcTS = new TCanvas("cmcVertexTS","cosThetaStar distribution from MC - Vertex cuts",50,50,550,550);  // cTS stands for cosThetaStar
	TCanvas *cmcVertexptPi = new TCanvas("cmcVertexptPi","pt_pi distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexptK = new TCanvas("cmcVertexptK","pt_K distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexcT = new TCanvas("cmcVertexcT","cT distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexdca = new TCanvas("cmcVertexdca","dca distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexd0pi = new TCanvas("cmcVertexd0pi","d0pi distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexd0K = new TCanvas("cmcVertexd0K","d0K distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexd0xd0 = new TCanvas("cmcVertexd0xd0","d0xd0 distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexpointing = new TCanvas("cmcVertexpointing","cosPointingAngle distribution from MC - Vertex cuts",50,50,550,550);
	TCanvas *cmcVertexphi = new TCanvas("cmcVertexphi","Phi distribution from MC - Vertex cuts",50,50,550,550);

	// MC Refit level
	TCanvas *cmcRefitpt = new TCanvas("cmcRefitpt","pt distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefity = new TCanvas("cmcRefity","y distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitcTS = new TCanvas("cmcRefitTS","cosThetaStar distribution from MC - Refit cuts",50,50,550,550);  // cTS stands for cosThetaStar
	TCanvas *cmcRefitptPi = new TCanvas("cmcRefitptPi","pt_pi distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitptK = new TCanvas("cmcRefitptK","pt_K distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitcT = new TCanvas("cmcRefitcT","cT distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitdca = new TCanvas("cmcRefitdca","dca distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitd0pi = new TCanvas("cmcRefitd0pi","d0pi distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitd0K = new TCanvas("cmcRefitd0K","d0K distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitd0xd0 = new TCanvas("cmcRefitd0xd0","d0xd0 distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitpointing = new TCanvas("cmcRefitpointing","cosPointingAngle distribution from MC - Refit cuts",50,50,550,550);
	TCanvas *cmcRefitphi = new TCanvas("cmcRefitphi","Phi distribution from MC - Refit cuts",50,50,550,550);

	// Reco-aod level
	TCanvas *cpt = new TCanvas("cpt","pt distribution from reco aod",50,50,550,550);
	TCanvas *cy = new TCanvas("cy","y distribution from reco aod",50,50,550,550);
	TCanvas *ccTS = new TCanvas("cTS","cosThetaStar distribution from reco aod",50,50,550,550); 
	TCanvas *cptPi = new TCanvas("cptPi","pt_pi distribution from reco aod",50,50,550,550);
	TCanvas *cptK = new TCanvas("cptK","pt_K distribution from reco aod",50,50,550,550);
	TCanvas *ccT = new TCanvas("ccT","cT distribution from reco aod",50,50,550,550);
	TCanvas *cdca = new TCanvas("cdca","dca distribution from reco aod",50,50,550,550);
	TCanvas *cd0pi = new TCanvas("cd0pi","d0pi distribution from reco aod",50,50,550,550);
	TCanvas *cd0K = new TCanvas("cd0K","d0K distribution from reco aod",50,50,550,550);
	TCanvas *cd0xd0 = new TCanvas("cd0xd0","d0xd0 distribution from reco aod",50,50,550,550);
	TCanvas *cpointing = new TCanvas("cpointing","cosPointingAngle distribution from reco aod",50,50,550,550);
	TCanvas *cphi = new TCanvas("cphi","Phi distribution from reco aod",50,50,550,550);

	// Reco-aod Acceptance level
	TCanvas *cAccpt = new TCanvas("cAccpt","pt distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAccy = new TCanvas("cAccy","y distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAcccTS = new TCanvas("cAccTS","cosThetaStar distribution from reco aod - Acceptance cuts",50,50,550,550); 
	TCanvas *cAccptPi = new TCanvas("cAccptPi","pt_pi distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAccptK = new TCanvas("cAccptK","pt_K distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAccdca = new TCanvas("cAccdca","dca distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAccd0pi = new TCanvas("cAccd0pi","d0pi distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAccd0K = new TCanvas("cAccd0K","d0K distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAccd0xd0 = new TCanvas("cAccd0xd0","d0xd0 distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAccpointing = new TCanvas("cAccpointing","cosPointingAngle distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAcccT = new TCanvas("cAcccT","cT distribution from reco aod - Acceptance cuts",50,50,550,550);
	TCanvas *cAccphi = new TCanvas("cAccphi","Phi distribution from reco aod - Acceptance cuts",50,50,550,550);

	// Reco-aod ITS Clusters level
	TCanvas *cITSClpt = new TCanvas("cITSClpt","pt distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSCly = new TCanvas("cITSCly","y distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSClcTS = new TCanvas("cITSClTS","cosThetaStar distribution from reco aod - ITS Clusters cuts",50,50,550,550); 
	TCanvas *cITSClptPi = new TCanvas("cITSClptPi","pt_pi distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSClptK = new TCanvas("cITSClptK","pt_K distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSClcT = new TCanvas("cITSClcT","cT distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSCldca = new TCanvas("cITSCldca","dca distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSCld0pi = new TCanvas("cITSCld0pi","d0pi distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSCld0K = new TCanvas("cITSCld0K","d0K distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSCld0xd0 = new TCanvas("cITSCld0xd0","d0xd0 distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSClpointing = new TCanvas("cITSClpointing","cosPointingAngle distribution from reco aod - ITS Clusters cuts",50,50,550,550);
	TCanvas *cITSClphi = new TCanvas("cITSClphi","Phi distribution from reco aod - ITS Clusters cuts",50,50,550,550);

	// Reco-aod PPR level
	TCanvas *cPPRpt = new TCanvas("cPPRpt","pt distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRy = new TCanvas("cPPRy","y distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRcTS = new TCanvas("cPPRTS","cosThetaStar distribution from reco aod - PPR cuts",50,50,550,550); 
	TCanvas *cPPRptPi = new TCanvas("cPPRptPi","pt_pi distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRptK = new TCanvas("cPPRptK","pt_K distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRcT = new TCanvas("cPPRcT","cT distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRdca = new TCanvas("cPPRdca","dca distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRd0pi = new TCanvas("cPPRd0pi","d0pi distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRd0K = new TCanvas("cPPRd0K","d0K distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRd0xd0 = new TCanvas("cPPRd0xd0","d0xd0 distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRpointing = new TCanvas("cPPRpointing","cosPointingAngle distribution from reco aod - PPR cuts",50,50,550,550);
	TCanvas *cPPRphi = new TCanvas("cPPRphi","Phi distribution from reco aod - PPR cuts",50,50,550,550);

	// pt, y, cosThetaStar for all steps
	cmc1->Divide(8,3);   

	cmc1->cd(1);
	TH1D *hMCpt1D = data->ShowProjection(ipt, stepGen);
	Double_t maxpt = hMCpt1D->GetMaximum();
	hMCpt1D->GetYaxis()->SetRangeUser(0,maxpt*1.2);
	hMCpt1D->Sumw2();
	hMCpt1D->SetMinimum(0.01);
	SetHistoProperties(hMCpt1D,2,3,2,20,2,3005);
	hMCpt1D->GetXaxis()->SetTitleOffset(1.2);
	hMCpt1D->GetXaxis()->SetTitle("p_{T} (GeV/c), MC data");
	hMCpt1D->Draw("hist");
	cmcpt->cd();
	cmcpt->SetLeftMargin(0.15);
	cmcpt->SetRightMargin(0.05);
	hMCpt1D->Draw("hist");
	hMCpt1D->Draw("err same");
	cmcpt->Update();
	
	cmc1->cd(2);
	TH1D *hMCAccpt1D = data->ShowProjection(ipt, stepGenAcc);
	maxpt = hMCAccpt1D->GetMaximum();
	hMCAccpt1D->GetYaxis()->SetRangeUser(0,maxpt*1.2);
	SetHistoProperties(hMCAccpt1D,8,3,8,20,8,3006);
	hMCAccpt1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAccpt1D->Sumw2();
	hMCAccpt1D->SetMinimum(0.01);
	hMCAccpt1D->GetXaxis()->SetTitle("p_{T} (GeV/c), MC - Acc cut");
	hMCAccpt1D->Draw("hist");
	cmcAccpt->cd();
	cmcAccpt->SetLeftMargin(0.15);
	cmcAccpt->SetRightMargin(0.05);
	hMCAccpt1D->Draw("hist");
	hMCAccpt1D->Draw("err same");
	cmcAccpt->Update();

	cmc1->cd(3);
	TH1D *hMCVertexpt1D = data->ShowProjection(ipt, stepGenVertex);
	maxpt = hMCVertexpt1D->GetMaximum();
	hMCVertexpt1D->GetYaxis()->SetRangeUser(0,maxpt*1.2);
	SetHistoProperties(hMCVertexpt1D,49,3,49,20,49,3006);
	hMCVertexpt1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexpt1D->Sumw2();
	hMCVertexpt1D->SetMinimum(0.01);
	hMCVertexpt1D->GetXaxis()->SetTitle("p_{T} (GeV/c), MC - Vertex cut");
	hMCVertexpt1D->Draw("hist");
	cmcVertexpt->cd();
	cmcVertexpt->SetLeftMargin(0.15);
	cmcVertexpt->SetRightMargin(0.05);
	hMCVertexpt1D->Draw("hist");
	hMCVertexpt1D->Draw("err same");
	cmcVertexpt->Update();

	cmc1->cd(4);
	TH1D *hMCRefitpt1D = data->ShowProjection(ipt, stepGenRefit);
	maxpt = hMCRefitpt1D->GetMaximum();
	hMCRefitpt1D->GetYaxis()->SetRangeUser(0,maxpt*1.2);
	SetHistoProperties(hMCRefitpt1D,42,3,42,20,42,3006);
	hMCRefitpt1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitpt1D->Sumw2();
	hMCRefitpt1D->SetMinimum(0.01);
	hMCRefitpt1D->GetXaxis()->SetTitle("p_{T} (GeV/c), MC - Refit cut");
	hMCRefitpt1D->Draw("hist");
	cmcRefitpt->cd();
	cmcRefitpt->SetLeftMargin(0.15);
	cmcRefitpt->SetRightMargin(0.05);
	hMCRefitpt1D->Draw("hist");
	hMCRefitpt1D->Draw("err same");
	cmcRefitpt->Update();

	cmc1->cd(5);
	TH1D *hRECpt1D = data->ShowProjection(ipt, stepRec);
	maxpt = hRECpt1D->GetMaximum();
	hRECpt1D->GetYaxis()->SetRangeUser(0,maxpt*1.2);
	SetHistoProperties(hRECpt1D,4,3,4,20,4,3004);
	hRECpt1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECpt1D->Sumw2();
	hRECpt1D->SetMinimum(0.01);
	hRECpt1D->GetXaxis()->SetTitle("p_{T} (GeV/c), AOD");
	hRECpt1D->Draw("hist");
	cpt->cd();
	cpt->SetLeftMargin(0.15);
	cpt->SetRightMargin(0.05);
	hRECpt1D->Draw("hist");
	hRECpt1D->Draw("err same");
	cpt->Update();

	cmc1->cd(6);
	TH1D *hRECAccpt1D = data->ShowProjection(ipt, stepRecAcc);
	maxpt = hRECAccpt1D->GetMaximum();
	hRECAccpt1D->GetYaxis()->SetRangeUser(0,maxpt*1.2);
	SetHistoProperties(hRECAccpt1D,38,3,38,20,38,3007);
	hRECAccpt1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccpt1D->Sumw2();
	hRECAccpt1D->SetMinimum(0.01);
	hRECAccpt1D->GetXaxis()->SetTitle("p_{T} (GeV/c), AOD - Acc cuts");
	hRECAccpt1D->Draw("hist");
	cAccpt->cd();
	cAccpt->SetLeftMargin(0.15);
	cAccpt->SetRightMargin(0.05);
	hRECAccpt1D->Draw("hist");
	hRECAccpt1D->Draw("err same");
	cAccpt->Update();

	cmc1->cd(7);
	TH1D *hRECITSClpt1D = data->ShowProjection(ipt, stepRecITSClus);
	maxpt = hRECITSClpt1D->GetMaximum();
	hRECITSClpt1D->GetYaxis()->SetRangeUser(0,maxpt*1.2);
	SetHistoProperties(hRECITSClpt1D,13,3,13,20,13,3019);
	hRECITSClpt1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSClpt1D->Sumw2();
	hRECITSClpt1D->SetMinimum(0.01);
	hRECITSClpt1D->GetXaxis()->SetTitle("p_{T} (GeV/c), AOD - ITS Cl cuts");
	hRECITSClpt1D->Draw("hist");
	cITSClpt->cd();
	cITSClpt->SetLeftMargin(0.15);
	cITSClpt->SetRightMargin(0.05);
	hRECITSClpt1D->Draw("hist");
	hRECITSClpt1D->Draw("err same");
	cITSClpt->Update();

	cmc1->cd(8);
	TH1D *hRECPPRpt1D = data->ShowProjection(ipt, stepRecPPR);
	maxpt = hRECPPRpt1D->GetMaximum();
	hRECPPRpt1D->GetYaxis()->SetRangeUser(0,maxpt*1.2);
	SetHistoProperties(hRECPPRpt1D,6,3,6,20,6,3019);
	hRECPPRpt1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRpt1D->Sumw2();
	hRECPPRpt1D->SetMinimum(0.01);
	hRECPPRpt1D->GetXaxis()->SetTitle("p_{T} (GeV/c), AOD - PPR cuts");
	hRECPPRpt1D->Draw("hist");
	cPPRpt->cd();
	cPPRpt->SetLeftMargin(0.15);
	cPPRpt->SetRightMargin(0.05);
	hRECPPRpt1D->Draw("hist");
	hRECPPRpt1D->Draw("err same");
	cPPRpt->Update();

	cmc1->cd(9);
	TH1D *hMCy1D = data->ShowProjection(iy, stepGen);
	Double_t maxy = hMCy1D->GetMaximum();
	hMCy1D->GetYaxis()->SetRangeUser(0,maxy*1.2);
	SetHistoProperties(hMCy1D,2,3,2,20,2,3005);
	hMCy1D->SetLineColor(2);
	hMCy1D->SetLineWidth(3);
	hMCy1D->SetMarkerColor(2);
	hMCy1D->SetFillColor(2);
	hMCy1D->SetFillStyle(3005);
	hMCy1D->SetMarkerStyle(20);
	hMCy1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCy1D->Sumw2();
	hMCy1D->SetMinimum(0.01);
	hMCy1D->GetXaxis()->SetTitle("y, MC data");
	hMCy1D->Draw("hist");
	cmcy->cd();
	cmcy->SetLeftMargin(0.15);
	cmcy->SetRightMargin(0.05);
	hMCy1D->Draw("hist");
	hMCy1D->Draw("err same");
	cmcy->Update();

	cmc1->cd(10);
	TH1D *hMCAccy1D = data->ShowProjection(iy, stepGenAcc);
	maxy = hMCAccy1D->GetMaximum();
	hMCAccy1D->GetYaxis()->SetRangeUser(0,maxy*1.2);
	SetHistoProperties(hMCAccy1D,8,3,8,20,8,3006);
	hMCAccy1D->GetXaxis()->SetTitleOffset(1.2);
	hMCAccy1D->Sumw2();
	hMCAccy1D->SetMinimum(0.01);
	hMCAccy1D->GetXaxis()->SetTitle("y, MC data - Acc cuts");
	hMCAccy1D->Draw("hist");
	cmcAccy->cd();
	cmcAccy->SetLeftMargin(0.15);
	cmcAccy->SetRightMargin(0.05);
	hMCAccy1D->Draw("hist");
	hMCAccy1D->Draw("err same");
	cmcy->Update();

	cmc1->cd(11);
	TH1D *hMCVertexy1D = data->ShowProjection(iy, stepGenVertex);
	maxy = hMCVertexy1D->GetMaximum();
	hMCVertexy1D->GetYaxis()->SetRangeUser(0,maxy*1.2);
	SetHistoProperties(hMCVertexy1D,49,3,49,20,49,3006);
	hMCVertexy1D->GetXaxis()->SetTitleOffset(1.2);
	hMCVertexy1D->Sumw2();
	hMCVertexy1D->SetMinimum(0.01);
	hMCVertexy1D->GetXaxis()->SetTitle("y, MC data - Vertex cuts");
	hMCVertexy1D->Draw("hist");
	cmcVertexy->cd();
	cmcVertexy->SetLeftMargin(0.15);
	cmcVertexy->SetRightMargin(0.05);
	hMCVertexy1D->Draw("hist");
	hMCVertexy1D->Draw("err same");
	cmcy->Update();

	cmc1->cd(12);
	TH1D *hMCRefity1D = data->ShowProjection(iy, stepGenRefit);
	maxy = hMCRefity1D->GetMaximum();
	hMCRefity1D->GetYaxis()->SetRangeUser(0,maxy*1.2);
	SetHistoProperties(hMCRefity1D,42,3,42,20,42,3006);
	hMCRefity1D->GetXaxis()->SetTitleOffset(1.2);
	hMCRefity1D->Sumw2();
	hMCRefity1D->SetMinimum(0.01);
	hMCRefity1D->GetXaxis()->SetTitle("y, MC data - Refit cuts");
	hMCRefity1D->Draw("hist");
	cmcRefity->cd();
	cmcRefity->SetLeftMargin(0.15);
	cmcRefity->SetRightMargin(0.05);
	hMCRefity1D->Draw("hist");
	hMCRefity1D->Draw("err same");
	cmcy->Update();

	cmc1->cd(13);
	TH1D *hRECy1D = data->ShowProjection(iy, stepRec);
	maxy = hRECy1D->GetMaximum();
	hRECy1D->GetYaxis()->SetRangeUser(0,maxy*1.2);
	SetHistoProperties(hRECy1D,4,3,4,20,4,3004);
	hRECy1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECy1D->Sumw2();
	hRECy1D->SetMinimum(0.01);
	hRECy1D->Draw("hist");
	cy->cd();
	cy->SetLeftMargin(0.15);
	cy->SetRightMargin(0.05);
	hRECy1D->GetXaxis()->SetTitle("y, AOD");
	hRECy1D->Draw("hist");
	hRECy1D->Draw("err same");
	cy->Update();

	cmc1->cd(14);
	TH1D *hRECAccy1D = data->ShowProjection(iy, stepRecAcc);
	maxy = hRECAccy1D->GetMaximum();
	hRECAccy1D->GetYaxis()->SetRangeUser(0,maxy*1.2);
	SetHistoProperties(hRECAccy1D,38,3,38,20,38,3007);
	hRECAccy1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccy1D->Sumw2();
	hRECAccy1D->SetMinimum(0.01);
	hRECAccy1D->Draw("hist");
	cAccy->cd();
	cAccy->SetLeftMargin(0.15);
	cAccy->SetRightMargin(0.05);
	hRECAccy1D->GetXaxis()->SetTitle("y, AOD - Acc cuts");
	hRECAccy1D->Draw("hist");
	hRECAccy1D->Draw("err same");
	cAccy->Update();

	cmc1->cd(15);
	TH1D *hRECITSCly1D = data->ShowProjection(iy, stepRecITSClus);
	maxy = hRECITSCly1D->GetMaximum();
	hRECITSCly1D->GetYaxis()->SetRangeUser(0,maxy*1.2);
	SetHistoProperties(hRECITSCly1D,13,3,13,20,13,3019);
	hRECITSCly1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSCly1D->Sumw2();
	hRECITSCly1D->SetMinimum(0.01);
	hRECITSCly1D->Draw("hist");
	cITSCly->cd();
	cITSCly->SetLeftMargin(0.15);
	cITSCly->SetRightMargin(0.05);
	hRECITSCly1D->GetXaxis()->SetTitle("y, AOD - ITS Cl cuts");
	hRECITSCly1D->Draw("hist");
	hRECITSCly1D->Draw("err same");
	cITSCly->Update();

	cmc1->cd(16);
	TH1D *hRECPPRy1D = data->ShowProjection(iy, stepRecPPR);
	maxy = hRECPPRy1D->GetMaximum();
	hRECPPRy1D->GetYaxis()->SetRangeUser(0,maxy*1.2);
	SetHistoProperties(hRECPPRy1D,6,3,6,20,6,3019);
	hRECPPRy1D->SetMarkerStyle(20);
	hRECPPRy1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRy1D->Sumw2();
	hRECPPRy1D->SetMinimum(0.01);
	hRECPPRy1D->Draw("hist");
	cPPRy->cd();
	cPPRy->SetLeftMargin(0.15);
	cPPRy->SetRightMargin(0.05);
	hRECPPRy1D->GetXaxis()->SetTitle("y, AOD - PPR cuts");
	hRECPPRy1D->Draw("hist");
	hRECPPRy1D->Draw("err same");
	cPPRy->Update();

	cmc1->cd(17);
	TH1D *hMCcTS1D = data->ShowProjection(icTS, stepGen);
	Double_t maxcTS = hMCcTS1D->GetMaximum();
	hMCcTS1D->GetYaxis()->SetRangeUser(0,maxcTS*1.2);
	SetHistoProperties(hMCcTS1D,2,3,2,20,2,3005);
	hMCcTS1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCcTS1D->Sumw2();
	hMCcTS1D->SetMinimum(0.01);
	hMCcTS1D->GetXaxis()->SetTitle("cosThetaStar, MC data");
	hMCcTS1D->Draw("hist");
	cmccTS->cd();
	cmccTS->SetLeftMargin(0.15);
	cmccTS->SetRightMargin(0.05);
	hMCcTS1D->Draw("hist");
	hMCcTS1D->Draw("err same");
	cmccTS->Update();

	cmc1->cd(18);
	TH1D *hMCAcccTS1D = data->ShowProjection(icTS, stepGenAcc);
	maxcTS = hMCAcccTS1D->GetMaximum();
	hMCAcccTS1D->GetYaxis()->SetRangeUser(0,maxcTS*1.2);
	SetHistoProperties(hMCAcccTS1D,8,3,8,20,8,3006);
	hMCAcccTS1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAcccTS1D->Sumw2();
	hMCAcccTS1D->SetMinimum(0.01);
	hMCAcccTS1D->GetXaxis()->SetTitle("cosThetaStar, MC data - Acc cuts");
	hMCAcccTS1D->Draw("hist");
	cmcAcccTS->cd();
	cmcAcccTS->SetLeftMargin(0.15);
	cmcAcccTS->SetRightMargin(0.05);
	hMCAcccTS1D->Draw("hist");
	hMCAcccTS1D->Draw("err same");
	cmcAcccTS->Update();

	cmc1->cd(19);
	TH1D *hMCVertexcTS1D = data->ShowProjection(icTS, stepGenVertex);
	maxcTS = hMCVertexcTS1D->GetMaximum();
	hMCVertexcTS1D->GetYaxis()->SetRangeUser(0,maxcTS*1.2);
	SetHistoProperties(hMCVertexcTS1D,49,3,49,20,49,3006);
	hMCVertexcTS1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexcTS1D->Sumw2();
	hMCVertexcTS1D->SetMinimum(0.01);
	hMCVertexcTS1D->GetXaxis()->SetTitle("cosThetaStar, MC data - Vertex cuts");
	hMCVertexcTS1D->Draw("hist");
	cmcVertexcTS->cd();
	cmcVertexcTS->SetLeftMargin(0.15);
	cmcVertexcTS->SetRightMargin(0.05);
	hMCVertexcTS1D->Draw("hist");
	hMCVertexcTS1D->Draw("err same");
	cmcVertexcTS->Update();

	cmc1->cd(20);
	TH1D *hMCRefitcTS1D = data->ShowProjection(icTS, stepGenRefit);
	maxcTS = hMCRefitcTS1D->GetMaximum();
	hMCRefitcTS1D->GetYaxis()->SetRangeUser(0,maxcTS*1.2);
	SetHistoProperties(hMCRefitcTS1D,42,3,42,20,42,3006);
	hMCRefitcTS1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitcTS1D->Sumw2();
	hMCRefitcTS1D->SetMinimum(0.01);
	hMCRefitcTS1D->GetXaxis()->SetTitle("cosThetaStar, MC data - Refit cuts");
	hMCRefitcTS1D->Draw("hist");
	cmcRefitcTS->cd();
	cmcRefitcTS->SetLeftMargin(0.15);
	cmcRefitcTS->SetRightMargin(0.05);
	hMCRefitcTS1D->Draw("hist");
	hMCRefitcTS1D->Draw("err same");
	cmcRefitcTS->Update();

	cmc1->cd(21);
	TH1D *hRECcTS1D = data->ShowProjection(icTS, stepRec);
	maxcTS = hRECcTS1D->GetMaximum();
	hRECcTS1D->GetYaxis()->SetRangeUser(0,maxcTS*1.2);
	SetHistoProperties(hRECcTS1D,4,3,4,20,4,3004);
	hRECcTS1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECcTS1D->Sumw2();
	hRECcTS1D->SetMinimum(0.01);
	hRECcTS1D->Draw("hist");
	ccTS->cd();
	ccTS->SetLeftMargin(0.15);
	ccTS->SetRightMargin(0.05);
	hRECcTS1D->GetXaxis()->SetTitle("cosThetaStar, AOD");
	hRECcTS1D->Draw("hist");
	hRECcTS1D->Draw("err same");
	ccTS->Update();

	cmc1->cd(22);
	TH1D *hRECAcccTS1D = data->ShowProjection(icTS, stepRecAcc);
	maxcTS = hRECAcccTS1D->GetMaximum();
	hRECAcccTS1D->GetYaxis()->SetRangeUser(0,maxcTS*1.2);
	SetHistoProperties(hRECAcccTS1D,38,3,38,20,38,3007);
	hRECAcccTS1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAcccTS1D->Sumw2();
	hRECAcccTS1D->SetMinimum(0.01);
	hRECAcccTS1D->Draw("hist");
	cAcccTS->cd();
	cAcccTS->SetLeftMargin(0.15);
	cAcccTS->SetRightMargin(0.05);
	hRECAcccTS1D->GetXaxis()->SetTitle("cosThetaStar, AOD - Acc cuts");
	hRECAcccTS1D->Draw("hist");
	hRECAcccTS1D->Draw("err same");
	cAcccTS->Update();

	cmc1->cd(23);
	TH1D *hRECITSClcTS1D = data->ShowProjection(icTS, stepRecITSClus);
	maxcTS = hRECITSClcTS1D->GetMaximum();
	hRECITSClcTS1D->GetYaxis()->SetRangeUser(0,maxcTS*1.2);
	SetHistoProperties(hRECITSClcTS1D,13,3,13,20,13,3019);
	hRECITSClcTS1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSClcTS1D->Sumw2();
	hRECITSClcTS1D->SetMinimum(0.01);
	hRECITSClcTS1D->Draw("hist");
	cITSClcTS->cd();
	cITSClcTS->SetLeftMargin(0.15);
	cITSClcTS->SetRightMargin(0.05);
	hRECITSClcTS1D->GetXaxis()->SetTitle("cosThetaStar, AOD - ITS Cl cuts");
	hRECITSClcTS1D->Draw("hist");
	hRECITSClcTS1D->Draw("err same");
	cITSClcTS->Update();

	cmc1->cd(24);
	TH1D *hRECPPRcTS1D = data->ShowProjection(icTS, stepRecPPR);
	maxcTS = hRECPPRcTS1D->GetMaximum();
	hRECPPRcTS1D->GetYaxis()->SetRangeUser(0,maxcTS*1.2);
	SetHistoProperties(hRECPPRcTS1D,6,3,6,20,6,3019);
	hRECPPRcTS1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRcTS1D->Sumw2();
	hRECPPRcTS1D->SetMinimum(0.01);
	hRECPPRcTS1D->Draw("hist");
	cPPRcTS->cd();
	cPPRcTS->SetLeftMargin(0.15);
	cPPRcTS->SetRightMargin(0.05);
	hRECPPRcTS1D->GetXaxis()->SetTitle("cosThetaStar, AOD - PPR cuts");
	hRECPPRcTS1D->Draw("hist");
	hRECPPRcTS1D->Draw("err same");
	cPPRcTS->Update();

	// ptPi, ptK, cT
	cmc2->Divide(8,3); 

	cmc2->cd(1);
	TH1D *hMCptPi1D = data->ShowProjection(iptPi, stepGen);
	Double_t maxptPi = hMCptPi1D->GetMaximum();
	hMCptPi1D->GetYaxis()->SetRangeUser(0,maxptPi*1.2);
	hMCptPi1D->Sumw2();
	SetHistoProperties(hMCptPi1D,2,3,2,20,2,3005);
	hMCptPi1D->SetMinimum(0.01);
	hMCptPi1D->GetXaxis()->SetTitleOffset(1.2);
	hMCptPi1D->GetXaxis()->SetTitle("p_{T, #pi} (GeV/c), MC data");
	hMCptPi1D->Draw("hist");
	cmcptPi->cd();
	cmcptPi->SetLeftMargin(0.15);
	cmcptPi->SetRightMargin(0.05);
	hMCptPi1D->Draw("hist");
	hMCptPi1D->Draw("err same");
	cmcptPi->Update();

	cmc2->cd(2);
	TH1D *hMCAccptPi1D = data->ShowProjection(iptPi, stepGenAcc);
	maxptPi = hMCAccptPi1D->GetMaximum();
	hMCAccptPi1D->GetYaxis()->SetRangeUser(0,maxptPi*1.2);
	hMCAccptPi1D->Sumw2();
	hMCAccptPi1D->SetMinimum(0.01);
	SetHistoProperties(hMCAccptPi1D,8,3,8,20,8,3006);
	hMCAccptPi1D->GetXaxis()->SetTitleOffset(1.2);
	hMCAccptPi1D->GetXaxis()->SetTitle("p_{T, #pi} (GeV/c), MC data - Acc cuts");
	hMCAccptPi1D->Draw("hist");
	cmcAccptPi->cd();
	cmcAccptPi->SetLeftMargin(0.15);
	cmcAccptPi->SetRightMargin(0.05);
	hMCAccptPi1D->Draw("hist");
	hMCAccptPi1D->Draw("err same");
	cmcAccptPi->Update();

	cmc2->cd(3);
	TH1D *hMCVertexptPi1D = data->ShowProjection(iptPi, stepGenVertex);
	maxptPi = hMCVertexptPi1D->GetMaximum();
	hMCVertexptPi1D->GetYaxis()->SetRangeUser(0,maxptPi*1.2);
	hMCVertexptPi1D->Sumw2();
	hMCVertexptPi1D->SetMinimum(0.01);
	SetHistoProperties(hMCVertexptPi1D,49,3,49,20,49,3006);
	hMCVertexptPi1D->GetXaxis()->SetTitleOffset(1.2);
	hMCVertexptPi1D->GetXaxis()->SetTitle("p_{T, #pi} (GeV/c), MC data - Vertex cuts");
	hMCVertexptPi1D->Draw("hist");
	cmcVertexptPi->cd();
	cmcVertexptPi->SetLeftMargin(0.15);
	cmcVertexptPi->SetRightMargin(0.05);
	hMCVertexptPi1D->Draw("hist");
	hMCVertexptPi1D->Draw("err same");
	cmcVertexptPi->Update();

	cmc2->cd(4);
	TH1D *hMCRefitptPi1D = data->ShowProjection(iptPi, stepGenRefit);
	maxptPi = hMCRefitptPi1D->GetMaximum();
	hMCRefitptPi1D->GetYaxis()->SetRangeUser(0,maxptPi*1.2);
	hMCRefitptPi1D->Sumw2();
	hMCRefitptPi1D->SetMinimum(0.01);
	SetHistoProperties(hMCRefitptPi1D,42,3,42,20,42,3006);
	hMCRefitptPi1D->GetXaxis()->SetTitleOffset(1.2);
	hMCRefitptPi1D->GetXaxis()->SetTitle("p_{T, #pi} (GeV/c), MC data - Refit cuts");
	hMCRefitptPi1D->Draw("hist");
	cmcRefitptPi->cd();
	cmcRefitptPi->SetLeftMargin(0.15);
	cmcRefitptPi->SetRightMargin(0.05);
	hMCRefitptPi1D->Draw("hist");
	hMCRefitptPi1D->Draw("err same");
	cmcRefitptPi->Update();

	cmc2->cd(5);
	TH1D *hRECptPi1D = data->ShowProjection(iptPi, stepRec);
	maxptPi = hRECptPi1D->GetMaximum();
	hRECptPi1D->GetYaxis()->SetRangeUser(0,maxptPi*1.2);
	SetHistoProperties(hRECptPi1D,4,3,4,20,4,3004);
	hRECptPi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECptPi1D->Sumw2();
	hRECptPi1D->SetMinimum(0.01);
	hRECptPi1D->GetXaxis()->SetTitle("p_{T, #pi} (GeV/c), AOD");
	hRECptPi1D->Draw("hist");
	cptPi->cd();
	cptPi->SetLeftMargin(0.15);
	cptPi->SetRightMargin(0.05);
	hRECptPi1D->Draw("hist");
	hRECptPi1D->Draw("err same");
	cptPi->Update();

	cmc2->cd(6);
	TH1D *hRECAccptPi1D = data->ShowProjection(iptPi, stepRecAcc);
	maxptPi = hRECAccptPi1D->GetMaximum();
	hRECAccptPi1D->GetYaxis()->SetRangeUser(0,maxptPi*1.2);
	SetHistoProperties(hRECAccptPi1D,38,3,38,20,38,3007);
	hRECAccptPi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccptPi1D->Sumw2();
	hRECAccptPi1D->SetMinimum(0.01);
	hRECAccptPi1D->GetXaxis()->SetTitle("p_{T, #pi} (GeV/c), AOD - Acc cuts");
	hRECAccptPi1D->Draw("hist");
	cAccptPi->cd();
	cAccptPi->SetLeftMargin(0.15);
	cAccptPi->SetRightMargin(0.05);
	hRECAccptPi1D->Draw("hist");
	hRECAccptPi1D->Draw("err same");
	cAccptPi->Update();

	cmc2->cd(7);
	TH1D *hRECITSClptPi1D = data->ShowProjection(iptPi, stepRecITSClus);
	maxptPi = hRECITSClptPi1D->GetMaximum();
	hRECITSClptPi1D->GetYaxis()->SetRangeUser(0,maxptPi*1.2);
	SetHistoProperties(hRECITSClptPi1D,13,3,13,20,13,3019);
	hRECITSClptPi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSClptPi1D->Sumw2();
	hRECITSClptPi1D->SetMinimum(0.01);
	hRECITSClptPi1D->GetXaxis()->SetTitle("p_{T, #pi} (GeV/c), AOD - ITS Cl cuts");
	hRECITSClptPi1D->Draw("hist");
	cITSClptPi->cd();
	cITSClptPi->SetLeftMargin(0.15);
	cITSClptPi->SetRightMargin(0.05);
	hRECITSClptPi1D->Draw("hist");
	hRECITSClptPi1D->Draw("err same");
	cITSClptPi->Update();

	cmc2->cd(8);
	TH1D *hRECPPRptPi1D = data->ShowProjection(iptPi, stepRecPPR);
	maxptPi = hRECPPRptPi1D->GetMaximum();
	hRECPPRptPi1D->GetYaxis()->SetRangeUser(0,maxptPi*1.2);
	SetHistoProperties(hRECPPRptPi1D,6,3,6,20,6,3019);
	hRECPPRptPi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRptPi1D->Sumw2();
	hRECPPRptPi1D->SetMinimum(0.01);
	hRECPPRptPi1D->GetXaxis()->SetTitle("p_{T, #pi} (GeV/c), AOD - PPR cuts");
	hRECPPRptPi1D->Draw("hist");
	cPPRptPi->cd();
	cPPRptPi->SetLeftMargin(0.15);
	cPPRptPi->SetRightMargin(0.05);
	hRECPPRptPi1D->Draw("hist");
	hRECPPRptPi1D->Draw("err same");
	cPPRptPi->Update();

	cmc2->cd(9);
	TH1D *hMCptK1D = data->ShowProjection(iptK, stepGen);
	Double_t maxptK = hMCptK1D->GetMaximum();
	hMCptK1D->GetYaxis()->SetRangeUser(0,maxptK*1.2);
	SetHistoProperties(hMCptK1D,2,3,2,20,2,3005);
	hMCptK1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCptK1D->Sumw2();
	hMCptK1D->SetMinimum(0.01);
	hMCptK1D->GetXaxis()->SetTitle("p_{T, K} (GeV/c), MC data");
	hMCptK1D->Draw("hist");
	cmcptK->cd();
	cmcptK->SetLeftMargin(0.15);
	cmcptK->SetRightMargin(0.05);
	hMCptK1D->Draw("hist");
	hMCptK1D->Draw("err same");
	cmcptK->Update();

	cmc2->cd(10);
	TH1D *hMCAccptK1D = data->ShowProjection(iptK, stepGenAcc);
	maxptK = hMCAccptK1D->GetMaximum();
	hMCAccptK1D->GetYaxis()->SetRangeUser(0,maxptK*1.2);
	SetHistoProperties(hMCAccptK1D,8,3,8,20,8,3006);
	hMCAccptK1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAccptK1D->Sumw2();
	hMCAccptK1D->SetMinimum(0.01);
	hMCAccptK1D->GetXaxis()->SetTitle("p_{T, K} (GeV/c), MC data - Acc cuts");
	hMCAccptK1D->Draw("hist");
	cmcAccptK->cd();
	cmcAccptK->SetLeftMargin(0.15);
	cmcAccptK->SetRightMargin(0.05);
	hMCAccptK1D->Draw("hist");
	hMCAccptK1D->Draw("err same");
	cmcAccptK->Update();

	cmc2->cd(11);
	TH1D *hMCVertexptK1D = data->ShowProjection(iptK, stepGenVertex);
	maxptK = hMCVertexptK1D->GetMaximum();
	hMCVertexptK1D->GetYaxis()->SetRangeUser(0,maxptK*1.2);
	SetHistoProperties(hMCVertexptK1D,49,3,49,20,49,3006);
	hMCVertexptK1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexptK1D->Sumw2();
	hMCVertexptK1D->SetMinimum(0.01);
	hMCVertexptK1D->GetXaxis()->SetTitle("p_{T, K} (GeV/c), MC data - Vertex cuts");
	hMCVertexptK1D->Draw("hist");
	cmcVertexptK->cd();
	cmcVertexptK->SetLeftMargin(0.15);
	cmcVertexptK->SetRightMargin(0.05);
	hMCVertexptK1D->Draw("hist");
	hMCVertexptK1D->Draw("err same");
	cmcVertexptK->Update();

	cmc2->cd(12);
	TH1D *hMCRefitptK1D = data->ShowProjection(iptK, stepGenRefit);
	maxptK = hMCRefitptK1D->GetMaximum();
	hMCRefitptK1D->GetYaxis()->SetRangeUser(0,maxptK*1.2);
	SetHistoProperties(hMCRefitptK1D,42,3,42,20,42,3006);
	hMCRefitptK1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitptK1D->Sumw2();
	hMCRefitptK1D->SetMinimum(0.01);
	hMCRefitptK1D->GetXaxis()->SetTitle("p_{T, K} (GeV/c), MC data - Refit cuts");
	hMCRefitptK1D->Draw("hist");
	cmcRefitptK->cd();
	cmcRefitptK->SetLeftMargin(0.15);
	cmcRefitptK->SetRightMargin(0.05);
	hMCRefitptK1D->Draw("hist");
	hMCRefitptK1D->Draw("err same");
	cmcRefitptK->Update();

	cmc2->cd(13);
	TH1D *hRECptK1D = data->ShowProjection(iptK, stepRec);
	maxptK = hRECptK1D->GetMaximum();
	hRECptK1D->GetYaxis()->SetRangeUser(0,maxptK*1.2);
	SetHistoProperties(hRECptK1D,4,3,4,20,4,3004);
	hRECptK1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECptK1D->Sumw2();
	hRECptK1D->SetMinimum(0.01);
	hRECptK1D->Draw("hist");
	cptK->cd();
	cptK->SetLeftMargin(0.15);
	cptK->SetRightMargin(0.05);
	hRECptK1D->GetXaxis()->SetTitle("p_{T, K} (GeV/c), AOD");
	hRECptK1D->Draw("hist");
	hRECptK1D->Draw("err same");
	cptK->Update();

	cmc2->cd(14);
	TH1D *hRECAccptK1D = data->ShowProjection(iptK, stepRecAcc);
	maxptK = hRECAccptK1D->GetMaximum();
	hRECAccptK1D->GetYaxis()->SetRangeUser(0,maxptK*1.2);
	SetHistoProperties(hRECAccptK1D,38,3,38,20,38,3007);
	hRECAccptK1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccptK1D->Sumw2();
	hRECAccptK1D->SetMinimum(0.01);
	hRECAccptK1D->Draw("hist");
	cAccptK->cd();
	cAccptK->SetLeftMargin(0.15);
	cAccptK->SetRightMargin(0.05);
	hRECAccptK1D->GetXaxis()->SetTitle("p_{T, K} (GeV/c), AOD - Acc cuts");
	hRECAccptK1D->Draw("hist");
	hRECAccptK1D->Draw("err same");
	cAccptK->Update();

	cmc2->cd(15);
	TH1D *hRECITSClptK1D = data->ShowProjection(iptK, stepRecITSClus);
	maxptK = hRECITSClptK1D->GetMaximum();
	hRECITSClptK1D->GetYaxis()->SetRangeUser(0,maxptK*1.2);
	SetHistoProperties(hRECITSClptK1D,13,3,13,20,13,3019);
	hRECITSClptK1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSClptK1D->Sumw2();
	hRECITSClptK1D->SetMinimum(0.01);
	hRECITSClptK1D->Draw("hist");
	cITSClptK->cd();
	cITSClptK->SetLeftMargin(0.15);
	cITSClptK->SetRightMargin(0.05);
	hRECITSClptK1D->GetXaxis()->SetTitle("p_{T, K} (GeV/c), AOD - ITS Cl cuts");
	hRECITSClptK1D->Draw("hist");
	hRECITSClptK1D->Draw("err same");
	cITSClptK->Update();

	cmc2->cd(16);
	TH1D *hRECPPRptK1D = data->ShowProjection(iptK, stepRecPPR);
	maxptK = hRECPPRptK1D->GetMaximum();
	hRECPPRptK1D->GetYaxis()->SetRangeUser(0,maxptK*1.2);
	SetHistoProperties(hRECPPRptK1D,6,3,6,20,6,3019);
	hRECPPRptK1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRptK1D->Sumw2();
	hRECPPRptK1D->SetMinimum(0.01);
	hRECPPRptK1D->Draw("hist");
	cPPRptK->cd();
	cPPRptK->SetLeftMargin(0.15);
	cPPRptK->SetRightMargin(0.05);
	hRECPPRptK1D->GetXaxis()->SetTitle("p_{T, K} (GeV/c), AOD - PPR cuts");
	hRECPPRptK1D->Draw("hist");
	hRECPPRptK1D->Draw("err same");
	cPPRptK->Update();

	cmc2->cd(17);
	TH1D *hMCcT1D = data->ShowProjection(icT, stepGen);
	Double_t maxcT = hMCcT1D->GetMaximum();
	hMCcT1D->GetYaxis()->SetRangeUser(0,maxcT*1.2);
	SetHistoProperties(hMCcT1D,2,3,2,20,2,3005);
	hMCcT1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCcT1D->Sumw2();
	hMCcT1D->SetMinimum(0.01);
	hMCcT1D->GetXaxis()->SetTitle("ct (#mum), MC data");
	hMCcT1D->Draw("hist");
	cmccT->cd();
	cmccT->SetLeftMargin(0.15);
	cmccT->SetRightMargin(0.05);
	hMCcT1D->Draw("hist");
	hMCcT1D->Draw("err same");
	cmccT->Update();

	cmc2->cd(18);
	TH1D *hMCAcccT1D = data->ShowProjection(icT, stepGenAcc);
	maxcT = hMCAcccT1D->GetMaximum();
	hMCAcccT1D->GetYaxis()->SetRangeUser(0,maxcT*1.2);
	SetHistoProperties(hMCAcccT1D,8,3,8,20,8,3006);
	hMCAcccT1D->GetXaxis()->SetTitleOffset(1.2);
	hMCAcccT1D->Sumw2();
	hMCAcccT1D->SetMinimum(0.01);
	hMCAcccT1D->GetXaxis()->SetTitle("ct (#mum), MC data - Acc cuts");
	hMCAcccT1D->Draw("hist");
	cmcAcccT->cd();
	cmcAcccT->SetLeftMargin(0.15);
	cmcAcccT->SetRightMargin(0.05);
	hMCAcccT1D->Draw("hist");
	hMCAcccT1D->Draw("err same");
	cmcAcccT->Update();

	cmc2->cd(19);
	TH1D *hMCVertexcT1D = data->ShowProjection(icT, stepGenVertex);
	maxcT = hMCVertexcT1D->GetMaximum();
	hMCVertexcT1D->GetYaxis()->SetRangeUser(0,maxcT*1.2);
	SetHistoProperties(hMCVertexcT1D,49,3,49,20,49,3006);
	hMCVertexcT1D->GetXaxis()->SetTitleOffset(1.2);
	hMCVertexcT1D->Sumw2();
	hMCVertexcT1D->SetMinimum(0.01);
	hMCVertexcT1D->GetXaxis()->SetTitle("ct (#mum), MC data - Vertex cuts");
	hMCVertexcT1D->Draw("hist");
	cmcVertexcT->cd();
	cmcVertexcT->SetLeftMargin(0.15);
	cmcVertexcT->SetRightMargin(0.05);
	hMCVertexcT1D->Draw("hist");
	hMCVertexcT1D->Draw("err same");
	cmcVertexcT->Update();

	cmc2->cd(20);
	TH1D *hMCRefitcT1D = data->ShowProjection(icT, stepGenRefit);
	maxcT = hMCRefitcT1D->GetMaximum();
	hMCRefitcT1D->GetYaxis()->SetRangeUser(0,maxcT*1.2);
	SetHistoProperties(hMCRefitcT1D,42,3,42,20,42,3006);
	hMCRefitcT1D->GetXaxis()->SetTitleOffset(1.2);
	hMCRefitcT1D->Sumw2();
	hMCRefitcT1D->SetMinimum(0.01);
	hMCRefitcT1D->GetXaxis()->SetTitle("ct (#mum), MC data - Refit cuts");
	hMCRefitcT1D->Draw("hist");
	cmcRefitcT->cd();
	cmcRefitcT->SetLeftMargin(0.15);
	cmcRefitcT->SetRightMargin(0.05);
	hMCRefitcT1D->Draw("hist");
	hMCRefitcT1D->Draw("err same");
	cmcRefitcT->Update();

	cmc2->cd(21);
	TH1D *hRECcT1D = data->ShowProjection(icT, stepRec);
	maxcT = hRECcT1D->GetMaximum();
	hRECcT1D->GetYaxis()->SetRangeUser(0,maxcT*1.2);
	SetHistoProperties(hRECcT1D,4,3,4,20,4,3004);
	hRECcT1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECcT1D->Sumw2();
	hRECcT1D->SetMinimum(0.01);
	hRECcT1D->Draw("hist");
	ccT->cd();
	ccT->SetLeftMargin(0.15);
	ccT->SetRightMargin(0.05);
	hRECcT1D->GetXaxis()->SetTitle("ct (#mum), AOD");
	hRECcT1D->Draw("hist");
	hRECcT1D->Draw("err same");
	ccT->Update();

	cmc2->cd(22);
	TH1D *hRECAcccT1D = data->ShowProjection(icT, stepRecAcc);
	maxcT = hRECAcccT1D->GetMaximum();
	hRECAcccT1D->GetYaxis()->SetRangeUser(0,maxcT*1.2);
	SetHistoProperties(hRECAcccT1D,38,3,38,20,38,3007);
	hRECAcccT1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAcccT1D->Sumw2();
	hRECAcccT1D->SetMinimum(0.01);
	hRECAcccT1D->Draw("hist");
	cAcccT->cd();
	cAcccT->SetLeftMargin(0.15);
	cAcccT->SetRightMargin(0.05);
	hRECAcccT1D->GetXaxis()->SetTitle("ct (#mum), AOD - Acc cuts");
	hRECAcccT1D->Draw("hist");
	hRECAcccT1D->Draw("err same");
	cAcccT->Update();

	cmc2->cd(23);
	TH1D *hRECITSClcT1D = data->ShowProjection(icT, stepRecITSClus);
	maxcT = hRECITSClcT1D->GetMaximum();
	hRECITSClcT1D->GetYaxis()->SetRangeUser(0,maxcT*1.2);
	SetHistoProperties(hRECITSClcT1D,13,3,13,20,13,3019);
	hRECITSClcT1D->GetXaxis()->SetTitleOffset(1.2);
	hRECITSClcT1D->Sumw2();
	hRECITSClcT1D->SetMinimum(0.01);
	hRECITSClcT1D->Draw("hist");
	cITSClcT->cd();
	cITSClcT->SetLeftMargin(0.15);
	cITSClcT->SetRightMargin(0.05);
	hRECITSClcT1D->GetXaxis()->SetTitle("ct (#mum), AOD - ITS Cl cuts");
	hRECITSClcT1D->Draw("hist");
	hRECITSClcT1D->Draw("err same");
	cITSClcT->Update();

	cmc2->cd(24);
	TH1D *hRECPPRcT1D = data->ShowProjection(icT, stepRecPPR);
	maxcT = hRECPPRcT1D->GetMaximum();
	hRECPPRcT1D->GetYaxis()->SetRangeUser(0,maxcT*1.2);
	SetHistoProperties(hRECPPRcT1D,6,3,6,20,6,3019);
	hRECPPRcT1D->GetXaxis()->SetTitleOffset(1.2);
	hRECPPRcT1D->Sumw2();
	hRECPPRcT1D->SetMinimum(0.01);
	hRECPPRcT1D->Draw("hist");
	cPPRcT->cd();
	cPPRcT->SetLeftMargin(0.15);
	cPPRcT->SetRightMargin(0.05);
	hRECPPRcT1D->GetXaxis()->SetTitle("ct (#mum), AOD - PPR cuts");
	hRECPPRcT1D->Draw("hist");
	hRECPPRcT1D->Draw("err same");
	cPPRcT->Update();

	//dca, d0pi, d0K
	cmc3->Divide(8,3);
	cmc3->cd(1);
	TH1D *hMCdca1D = data->ShowProjection(idca, stepGen);
	Double_t maxdca = hMCdca1D->GetMaximum();
	hMCdca1D->GetYaxis()->SetRangeUser(0,maxdca*1.2);
	hMCdca1D->Sumw2();
	hMCdca1D->SetMinimum(0.01);
	SetHistoProperties(hMCdca1D,2,3,2,20,2,3005);
	hMCdca1D->GetXaxis()->SetTitleOffset(1.2);
	hMCdca1D->GetXaxis()->SetTitle("dca (#mum), MC data");
	hMCdca1D->Draw("hist");
	cmcdca->cd();
	cmcdca->SetLeftMargin(0.15);
	cmcdca->SetRightMargin(0.05);
	hMCdca1D->Draw("hist");
	hMCdca1D->Draw("err same");
	cmcdca->Update();

	cmc3->cd(2);
	TH1D *hMCAccdca1D = data->ShowProjection(idca, stepGenAcc);
	maxdca = hMCAccdca1D->GetMaximum();
	hMCAccdca1D->GetYaxis()->SetRangeUser(0,maxdca*1.2);
	SetHistoProperties(hMCAccdca1D,8,3,8,20,8,3006);
	hMCAccdca1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAccdca1D->Sumw2();
	hMCAccdca1D->SetMinimum(0.01);
	hMCAccdca1D->GetXaxis()->SetTitle("p_{T} (GeV/c), MC - Acc cut");
	hMCAccdca1D->Draw("hist");
	cmcAccdca->cd();
	cmcAccdca->SetLeftMargin(0.15);
	cmcAccdca->SetRightMargin(0.05);
	hMCAccdca1D->Draw("hist");
	hMCAccdca1D->Draw("err same");
	cmcAccdca->Update();

	cmc3->cd(3);
	TH1D *hMCVertexdca1D = data->ShowProjection(idca, stepGenVertex);
	maxdca = hMCVertexdca1D->GetMaximum();
	hMCVertexdca1D->GetYaxis()->SetRangeUser(0,maxdca*1.2);
	SetHistoProperties(hMCVertexdca1D,49,3,49,20,49,3006);
	hMCVertexdca1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexdca1D->Sumw2();
	hMCVertexdca1D->SetMinimum(0.01);
	hMCVertexdca1D->GetXaxis()->SetTitle("p_{T} (GeV/c), MC - Vertex cut");
	hMCVertexdca1D->Draw("hist");
	cmcVertexdca->cd();
	cmcVertexdca->SetLeftMargin(0.15);
	cmcVertexdca->SetRightMargin(0.05);
	hMCVertexdca1D->Draw("hist");
	hMCVertexdca1D->Draw("err same");
	cmcVertexdca->Update();

	cmc3->cd(4);
	TH1D *hMCRefitdca1D = data->ShowProjection(idca, stepGenRefit);
	maxdca = hMCRefitdca1D->GetMaximum();
	hMCRefitdca1D->GetYaxis()->SetRangeUser(0,maxdca*1.2);
	SetHistoProperties(hMCRefitdca1D,42,3,42,20,42,3006);
	hMCRefitdca1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitdca1D->Sumw2();
	hMCRefitdca1D->SetMinimum(0.01);
	hMCRefitdca1D->GetXaxis()->SetTitle("p_{T} (GeV/c), MC - Refit cut");
	hMCRefitdca1D->Draw("hist");
	cmcRefitdca->cd();
	cmcRefitdca->SetLeftMargin(0.15);
	cmcRefitdca->SetRightMargin(0.05);
	hMCRefitdca1D->Draw("hist");
	hMCRefitdca1D->Draw("err same");
	cmcRefitdca->Update();

	cmc3->cd(5);
	TH1D *hRECdca1D = data->ShowProjection(idca, stepRec);
	maxdca = hRECdca1D->GetMaximum();
	hRECdca1D->GetYaxis()->SetRangeUser(0,maxdca*1.2);
	SetHistoProperties(hRECdca1D,4,3,4,20,4,3004);
	hRECdca1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECdca1D->Sumw2();
	hRECdca1D->SetMinimum(0.01);
	hRECdca1D->GetXaxis()->SetTitle("dca (#mum), AOD");
	hRECdca1D->Draw("hist");
	cdca->cd();
	cdca->SetLeftMargin(0.15);
	cdca->SetRightMargin(0.05);
	hRECdca1D->Draw("hist");
	hRECdca1D->Draw("err same");
	cdca->Update();

	cmc3->cd(6);
	TH1D *hRECAccdca1D = data->ShowProjection(idca, stepRecAcc);
	maxdca = hRECAccdca1D->GetMaximum();
	hRECAccdca1D->GetYaxis()->SetRangeUser(0,maxdca*1.2);
	SetHistoProperties(hRECAccdca1D,38,3,38,20,38,3007);
	hRECAccdca1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccdca1D->Sumw2();
	hRECAccdca1D->SetMinimum(0.01);
	hRECAccdca1D->GetXaxis()->SetTitle("dca (#mum), AOD - Acc cuts");
	hRECAccdca1D->Draw("hist");
	cAccdca->cd();
	cAccdca->SetLeftMargin(0.15);
	cAccdca->SetRightMargin(0.05);
	hRECAccdca1D->Draw("hist");
	hRECAccdca1D->Draw("err same");
	cAccdca->Update();

	cmc3->cd(6);
	TH1D *hRECITSCldca1D = data->ShowProjection(idca, stepRecITSClus);
	maxdca = hRECITSCldca1D->GetMaximum();
	hRECITSCldca1D->GetYaxis()->SetRangeUser(0,maxdca*1.2);
	SetHistoProperties(hRECITSCldca1D,13,3,13,20,13,3019);
	hRECITSCldca1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSCldca1D->Sumw2();
	hRECITSCldca1D->SetMinimum(0.01);
	hRECITSCldca1D->GetXaxis()->SetTitle("dca (#mum), AOD - ITS Cl cuts");
	hRECITSCldca1D->Draw("hist");
	cITSCldca->cd();
	cITSCldca->SetLeftMargin(0.15);
	cITSCldca->SetRightMargin(0.05);
	hRECITSCldca1D->Draw("hist");
	hRECITSCldca1D->Draw("err same");
	cITSCldca->Update();

	cmc3->cd(7);
	TH1D *hRECPPRdca1D = data->ShowProjection(idca, stepRecPPR);
	maxdca = hRECPPRdca1D->GetMaximum();
	hRECPPRdca1D->GetYaxis()->SetRangeUser(0,maxdca*1.2);
	SetHistoProperties(hRECPPRdca1D,6,3,6,20,6,3019);
	hRECPPRdca1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRdca1D->Sumw2();
	hRECPPRdca1D->SetMinimum(0.01);
	hRECPPRdca1D->GetXaxis()->SetTitle("dca (#mum), AOD - PPR cuts");
	hRECPPRdca1D->Draw("hist");
	cPPRdca->cd();
	cPPRdca->SetLeftMargin(0.15);
	cPPRdca->SetRightMargin(0.05);
	hRECPPRdca1D->Draw("hist");
	hRECPPRdca1D->Draw("err same");
	cPPRdca->Update();

	cmc3->cd(8);
	TH1D *hMCd0pi1D = data->ShowProjection(id0pi, stepGen);
	Double_t maxd0pi = hMCd0pi1D->GetMaximum();
	hMCd0pi1D->GetYaxis()->SetRangeUser(0,maxd0pi*1.2);
	hMCd0pi1D->Sumw2();
	hMCd0pi1D->SetMinimum(0.01);
	SetHistoProperties(hMCd0pi1D,2,3,2,20,2,3005);
	hMCd0pi1D->GetXaxis()->SetTitleOffset(1.2);
	hMCd0pi1D->GetXaxis()->SetTitle("d0_{#pi} (#mum), MC data");
	hMCd0pi1D->Draw("hist");
	cmcd0pi->cd();
	cmcd0pi->SetLeftMargin(0.15);
	cmcd0pi->SetRightMargin(0.05);
	hMCd0pi1D->Draw("hist");
	hMCd0pi1D->Draw("err same");
	cmcd0pi->Update();

	cmc3->cd(9);
	TH1D *hMCAccd0pi1D = data->ShowProjection(id0pi, stepGenAcc);
	maxd0pi = hMCAccd0pi1D->GetMaximum();
	hMCAccd0pi1D->GetYaxis()->SetRangeUser(0,maxd0pi*1.2);
	SetHistoProperties(hMCAccd0pi1D,8,3,8,20,8,3006);
	hMCAccd0pi1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAccd0pi1D->Sumw2();
	hMCAccd0pi1D->SetMinimum(0.01);
	hMCAccd0pi1D->GetXaxis()->SetTitle("d0_{#pi} (#mum), MC - Acc cut");
	hMCAccd0pi1D->Draw("hist");
	cmcAccd0pi->cd();
	cmcAccd0pi->SetLeftMargin(0.15);
	cmcAccd0pi->SetRightMargin(0.05);
	hMCAccd0pi1D->Draw("hist");
	hMCAccd0pi1D->Draw("err same");
	cmcAccd0pi->Update();

	cmc3->cd(10);
	TH1D *hMCVertexd0pi1D = data->ShowProjection(id0pi, stepGenVertex);
	maxd0pi = hMCVertexd0pi1D->GetMaximum();
	hMCVertexd0pi1D->GetYaxis()->SetRangeUser(0,maxd0pi*1.2);
	SetHistoProperties(hMCVertexd0pi1D,49,3,49,20,49,3006);
	hMCVertexd0pi1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexd0pi1D->Sumw2();
	hMCVertexd0pi1D->SetMinimum(0.01);
	hMCVertexd0pi1D->GetXaxis()->SetTitle("d0_{#pi} (#mum), MC - Vertex cut");
	hMCVertexd0pi1D->Draw("hist");
	cmcVertexd0pi->cd();
	cmcVertexd0pi->SetLeftMargin(0.15);
	cmcVertexd0pi->SetRightMargin(0.05);
	hMCVertexd0pi1D->Draw("hist");
	hMCVertexd0pi1D->Draw("err same");
	cmcVertexd0pi->Update();

	cmc3->cd(11);
	TH1D *hMCRefitd0pi1D = data->ShowProjection(id0pi, stepGenRefit);
	maxd0pi = hMCRefitd0pi1D->GetMaximum();
	hMCRefitd0pi1D->GetYaxis()->SetRangeUser(0,maxd0pi*1.2);
	SetHistoProperties(hMCRefitd0pi1D,42,3,42,20,42,3006);
	hMCRefitd0pi1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitd0pi1D->Sumw2();
	hMCRefitd0pi1D->SetMinimum(0.01);
	hMCRefitd0pi1D->GetXaxis()->SetTitle("d0_{#pi} (#mum), MC - Refit cut");
	hMCRefitd0pi1D->Draw("hist");
	cmcRefitd0pi->cd();
	cmcRefitd0pi->SetLeftMargin(0.15);
	cmcRefitd0pi->SetRightMargin(0.05);
	hMCRefitd0pi1D->Draw("hist");
	hMCRefitd0pi1D->Draw("err same");
	cmcRefitd0pi->Update();

	cmc3->cd(12);
	TH1D *hRECd0pi1D = data->ShowProjection(id0pi, stepRec);
	maxd0pi = hRECd0pi1D->GetMaximum();
	hRECd0pi1D->GetYaxis()->SetRangeUser(0,maxd0pi*1.2);
	SetHistoProperties(hRECd0pi1D,4,3,4,20,4,3004);
	hRECd0pi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECd0pi1D->Sumw2();
	hRECd0pi1D->SetMinimum(0.01);
	hRECd0pi1D->GetXaxis()->SetTitle("d0_{#pi} (#mum), AOD");
	hRECd0pi1D->Draw("hist");
	cd0pi->cd();
	cd0pi->SetLeftMargin(0.15);
	cd0pi->SetRightMargin(0.05);
	hRECd0pi1D->Draw("hist");
	hRECd0pi1D->Draw("err same");
	cd0pi->Update();

	cmc3->cd(13);
	TH1D *hRECAccd0pi1D = data->ShowProjection(id0pi, stepRecAcc);
	maxd0pi = hRECAccd0pi1D->GetMaximum();
	hRECAccd0pi1D->GetYaxis()->SetRangeUser(0,maxd0pi*1.2);
	SetHistoProperties(hRECAccd0pi1D,38,3,38,20,38,3007);
	hRECAccd0pi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccd0pi1D->Sumw2();
	hRECAccd0pi1D->SetMinimum(0.01);
	hRECAccd0pi1D->GetXaxis()->SetTitle("d0_{#pi} (#mum), AOD - Acc cuts");
	hRECAccd0pi1D->Draw("hist");
	cAccd0pi->cd();
	cAccd0pi->SetLeftMargin(0.15);
	cAccd0pi->SetRightMargin(0.05);
	hRECAccd0pi1D->Draw("hist");
	hRECAccd0pi1D->Draw("err same");
	cAccd0pi->Update();

	cmc3->cd(14);
	TH1D *hRECITSCld0pi1D = data->ShowProjection(id0pi, stepRecITSClus);
	maxd0pi = hRECITSCld0pi1D->GetMaximum();
	hRECITSCld0pi1D->GetYaxis()->SetRangeUser(0,maxd0pi*1.2);
	SetHistoProperties(hRECITSCld0pi1D,13,3,13,20,13,3019);
	hRECITSCld0pi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSCld0pi1D->Sumw2();
	hRECITSCld0pi1D->SetMinimum(0.01);
	hRECITSCld0pi1D->GetXaxis()->SetTitle("d0_{#pi} (#mum), AOD - ITS Cl cuts");
	hRECITSCld0pi1D->Draw("hist");
	cITSCld0pi->cd();
	cITSCld0pi->SetLeftMargin(0.15);
	cITSCld0pi->SetRightMargin(0.05);
	hRECITSCld0pi1D->Draw("hist");
	hRECITSCld0pi1D->Draw("err same");
	cITSCld0pi->Update();

	cmc3->cd(15);
	TH1D *hRECPPRd0pi1D = data->ShowProjection(id0pi, stepRecPPR);
	maxd0pi = hRECPPRd0pi1D->GetMaximum();
	hRECPPRd0pi1D->GetYaxis()->SetRangeUser(0,maxd0pi*1.2);
	SetHistoProperties(hRECPPRd0pi1D,6,3,6,20,6,3019);
	hRECPPRd0pi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRd0pi1D->Sumw2();
	hRECPPRd0pi1D->SetMinimum(0.01);
	hRECPPRd0pi1D->GetXaxis()->SetTitle("d0_{#pi} (#mum), AOD - PPR cuts");
	hRECPPRd0pi1D->Draw("hist");
	cPPRd0pi->cd();
	cPPRd0pi->SetLeftMargin(0.15);
	cPPRd0pi->SetRightMargin(0.05);
	hRECPPRd0pi1D->Draw("hist");
	hRECPPRd0pi1D->Draw("err same");
	cPPRd0pi->Update();

	cmc3->cd(16);
	TH1D *hMCd0K1D = data->ShowProjection(id0K, stepGen);
	Double_t maxd0K = hMCd0K1D->GetMaximum();
	hMCd0K1D->GetYaxis()->SetRangeUser(0,maxd0K*1.2);
	hMCd0K1D->Sumw2();
	hMCd0K1D->SetMinimum(0.01);
	SetHistoProperties(hMCd0K1D,2,3,2,20,2,3005);
	hMCd0K1D->GetXaxis()->SetTitleOffset(1.2);
	hMCd0K1D->GetXaxis()->SetTitle("d0_{K} (#mum), MC data");
	hMCd0K1D->Draw("hist");
	cmcd0K->cd();
	cmcd0K->SetLeftMargin(0.15);
	cmcd0K->SetRightMargin(0.05);
	hMCd0K1D->Draw("hist");
	hMCd0K1D->Draw("err same");
	cmcd0K->Update();

	cmc3->cd(17);
	TH1D *hMCAccd0K1D = data->ShowProjection(id0K, stepGenAcc);
	maxd0K = hMCAccd0K1D->GetMaximum();
	hMCAccd0K1D->GetYaxis()->SetRangeUser(0,maxd0K*1.2);
	SetHistoProperties(hMCAccd0K1D,8,3,8,20,8,3006);
	hMCAccd0K1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAccd0K1D->Sumw2();
	hMCAccd0K1D->SetMinimum(0.01);
	hMCAccd0K1D->GetXaxis()->SetTitle("d0_{K} (#mum), MC - Acc cut");
	hMCAccd0K1D->Draw("hist");
	cmcAccd0K->cd();
	cmcAccd0K->SetLeftMargin(0.15);
	cmcAccd0K->SetRightMargin(0.05);
	hMCAccd0K1D->Draw("hist");
	hMCAccd0K1D->Draw("err same");
	cmcAccd0K->Update();

	cmc3->cd(18);
	TH1D *hMCVertexd0K1D = data->ShowProjection(id0K, stepGenVertex);
	maxd0K = hMCVertexd0K1D->GetMaximum();
	hMCVertexd0K1D->GetYaxis()->SetRangeUser(0,maxd0K*1.2);
	SetHistoProperties(hMCVertexd0K1D,49,3,49,20,49,3006);
	hMCVertexd0K1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexd0K1D->Sumw2();
	hMCVertexd0K1D->SetMinimum(0.01);
	hMCVertexd0K1D->GetXaxis()->SetTitle("d0_{K} (#mum), MC - Vertex cut");
	hMCVertexd0K1D->Draw("hist");
	cmcVertexd0K->cd();
	cmcVertexd0K->SetLeftMargin(0.15);
	cmcVertexd0K->SetRightMargin(0.05);
	hMCVertexd0K1D->Draw("hist");
	hMCVertexd0K1D->Draw("err same");
	cmcVertexd0K->Update();

	cmc3->cd(20);
	TH1D *hMCRefitd0K1D = data->ShowProjection(id0K, stepGenRefit);
	maxd0K = hMCRefitd0K1D->GetMaximum();
	hMCRefitd0K1D->GetYaxis()->SetRangeUser(0,maxd0K*1.2);
	SetHistoProperties(hMCRefitd0K1D,42,3,42,20,42,3006);
	hMCRefitd0K1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitd0K1D->Sumw2();
	hMCRefitd0K1D->SetMinimum(0.01);
	hMCRefitd0K1D->GetXaxis()->SetTitle("d0_{K} (#mum), MC - Refit cut");
	hMCRefitd0K1D->Draw("hist");
	cmcRefitd0K->cd();
	cmcRefitd0K->SetLeftMargin(0.15);
	cmcRefitd0K->SetRightMargin(0.05);
	hMCRefitd0K1D->Draw("hist");
	hMCRefitd0K1D->Draw("err same");
	cmcRefitd0K->Update();

	cmc3->cd(21);
	TH1D *hRECd0K1D = data->ShowProjection(id0K, stepRec);
	maxd0K = hRECd0K1D->GetMaximum();
	hRECd0K1D->GetYaxis()->SetRangeUser(0,maxd0K*1.2);
	SetHistoProperties(hRECd0K1D,4,3,4,20,4,3004);
	hRECd0K1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECd0K1D->Sumw2();
	hRECd0K1D->SetMinimum(0.01);
	hRECd0K1D->GetXaxis()->SetTitle("d0_{K} (#mum), AOD");
	hRECd0K1D->Draw("hist");
	cd0K->cd();
	cd0K->SetLeftMargin(0.15);
	cd0K->SetRightMargin(0.05);
	hRECd0K1D->Draw("hist");
	hRECd0K1D->Draw("err same");
	cd0K->Update();

	cmc3->cd(22);
	TH1D *hRECAccd0K1D = data->ShowProjection(id0K, stepRecAcc);
	maxd0K = hRECAccd0K1D->GetMaximum();
	hRECAccd0K1D->GetYaxis()->SetRangeUser(0,maxd0K*1.2);
	SetHistoProperties(hRECAccd0K1D,38,3,38,20,38,3007);
	hRECAccd0K1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccd0K1D->Sumw2();
	hRECAccd0K1D->SetMinimum(0.01);
	hRECAccd0K1D->GetXaxis()->SetTitle("d0_{K} (#mum), AOD - Acc cuts");
	hRECAccd0K1D->Draw("hist");
	cAccd0K->cd();
	cAccd0K->SetLeftMargin(0.15);
	cAccd0K->SetRightMargin(0.05);
	hRECAccd0K1D->Draw("hist");
	hRECAccd0K1D->Draw("err same");
	cAccd0K->Update();

	cmc3->cd(23);
	TH1D *hRECITSCld0K1D = data->ShowProjection(id0K, stepRecITSClus);
	maxd0K = hRECITSCld0K1D->GetMaximum();
	hRECITSCld0K1D->GetYaxis()->SetRangeUser(0,maxd0K*1.2);
	SetHistoProperties(hRECITSCld0K1D,13,3,13,20,13,3019);
	hRECITSCld0K1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSCld0K1D->Sumw2();
	hRECITSCld0K1D->SetMinimum(0.01);
	hRECITSCld0K1D->GetXaxis()->SetTitle("d0_{K} (#mum), AOD - ITS Cl cuts");
	hRECITSCld0K1D->Draw("hist");
	cITSCld0K->cd();
	cITSCld0K->SetLeftMargin(0.15);
	cITSCld0K->SetRightMargin(0.05);
	hRECITSCld0K1D->Draw("hist");
	hRECITSCld0K1D->Draw("err same");
	cITSCld0K->Update();

	cmc3->cd(24);
	TH1D *hRECPPRd0K1D = data->ShowProjection(id0K, stepRecPPR);
	maxd0K = hRECPPRd0K1D->GetMaximum();
	hRECPPRd0K1D->GetYaxis()->SetRangeUser(0,maxd0K*1.2);
	SetHistoProperties(hRECPPRd0K1D,6,3,6,20,6,3019);
	hRECPPRd0K1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRd0K1D->Sumw2();
	hRECPPRd0K1D->SetMinimum(0.01);
	hRECPPRd0K1D->GetXaxis()->SetTitle("d0_{K} (#mum), AOD - PPR cuts");
	hRECPPRd0K1D->Draw("hist");
	cPPRd0K->cd();
	cPPRd0K->SetLeftMargin(0.15);
	cPPRd0K->SetRightMargin(0.05);
	hRECPPRd0K1D->Draw("hist");
	hRECPPRd0K1D->Draw("err same");
	cPPRd0K->Update();

	//d0xd0, cosPointingAngle, Phi
	cmc4->Divide(8,3);
	cmc4->cd(1);
	TH1D *hMCd0xd01D = data->ShowProjection(id0xd0, stepGen);
	Double_t maxd0xd0 = hMCd0xd01D->GetMaximum();
	hMCd0xd01D->GetYaxis()->SetRangeUser(0,maxd0xd0*1.2);
	hMCd0xd01D->Sumw2();
	hMCd0xd01D->SetMinimum(0.01);
	SetHistoProperties(hMCd0xd01D,2,3,2,20,2,3005);
	hMCd0xd01D->GetXaxis()->SetTitleOffset(1.2);
	hMCd0xd01D->GetXaxis()->SetTitle("d0_{#pi}xd0_{K} (#mum^2), MC data");
	hMCd0xd01D->Draw("hist");
	cmcd0xd0->cd();
	cmcd0xd0->SetLeftMargin(0.15);
	cmcd0xd0->SetRightMargin(0.05);
	hMCd0xd01D->Draw("hist");
	hMCd0xd01D->Draw("err same");
	cmcd0xd0->Update();

	cmc4->cd(2);
	TH1D *hMCAccd0xd01D = data->ShowProjection(id0xd0, stepGenAcc);
	maxd0xd0 = hMCAccd0xd01D->GetMaximum();
	hMCAccd0xd01D->GetYaxis()->SetRangeUser(0,maxd0xd0*1.2);
	SetHistoProperties(hMCAccd0xd01D,8,3,8,20,8,3006);
	hMCAccd0xd01D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAccd0xd01D->Sumw2();
	hMCAccd0xd01D->SetMinimum(0.01);
	hMCAccd0xd01D->GetXaxis()->SetTitle("d0_{#pi}xd0_{K} (#mum^2), MC - Acc cut");
	hMCAccd0xd01D->Draw("hist");
	cmcAccd0xd0->cd();
	cmcAccd0xd0->SetLeftMargin(0.15);
	cmcAccd0xd0->SetRightMargin(0.05);
	hMCAccd0xd01D->Draw("hist");
	hMCAccd0xd01D->Draw("err same");
	cmcAccd0xd0->Update();

	cmc4->cd(3);
	TH1D *hMCVertexd0xd01D = data->ShowProjection(id0xd0, stepGenVertex);
	maxd0xd0 = hMCVertexd0xd01D->GetMaximum();
	hMCVertexd0xd01D->GetYaxis()->SetRangeUser(0,maxd0xd0*1.2);
	SetHistoProperties(hMCVertexd0xd01D,49,3,49,20,49,3006);
	hMCVertexd0xd01D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexd0xd01D->Sumw2();
	hMCVertexd0xd01D->SetMinimum(0.01);
	hMCVertexd0xd01D->GetXaxis()->SetTitle("d0_{#pi}xd0_{K} (#mum^2), MC - Vertex cut");
	hMCVertexd0xd01D->Draw("hist");
	cmcVertexd0xd0->cd();
	cmcVertexd0xd0->SetLeftMargin(0.15);
	cmcVertexd0xd0->SetRightMargin(0.05);
	hMCVertexd0xd01D->Draw("hist");
	hMCVertexd0xd01D->Draw("err same");
	cmcVertexd0xd0->Update();

	cmc4->cd(4);
	TH1D *hMCRefitd0xd01D = data->ShowProjection(id0xd0, stepGenRefit);
	maxd0xd0 = hMCRefitd0xd01D->GetMaximum();
	hMCRefitd0xd01D->GetYaxis()->SetRangeUser(0,maxd0xd0*1.2);
	SetHistoProperties(hMCRefitd0xd01D,42,3,42,20,42,3006);
	hMCRefitd0xd01D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitd0xd01D->Sumw2();
	hMCRefitd0xd01D->SetMinimum(0.01);
	hMCRefitd0xd01D->GetXaxis()->SetTitle("d0_{#pi}xd0_{K} (#mum^2), MC - Refit cut");
	hMCRefitd0xd01D->Draw("hist");
	cmcRefitd0xd0->cd();
	cmcRefitd0xd0->SetLeftMargin(0.15);
	cmcRefitd0xd0->SetRightMargin(0.05);
	hMCRefitd0xd01D->Draw("hist");
	hMCRefitd0xd01D->Draw("err same");
	cmcRefitd0xd0->Update();

	cmc4->cd(5);
	TH1D *hRECd0xd01D = data->ShowProjection(id0xd0, stepRec);
	maxd0xd0 = hRECd0xd01D->GetMaximum();
	hRECd0xd01D->GetYaxis()->SetRangeUser(0,maxd0xd0*1.2);
	SetHistoProperties(hRECd0xd01D,4,3,4,20,4,3004);
	hRECd0xd01D->GetXaxis()->SetTitleOffset(1.2);;
	hRECd0xd01D->Sumw2();
	hRECd0xd01D->SetMinimum(0.01);
	hRECd0xd01D->GetXaxis()->SetTitle("d0_{#pi}xd0_{K} (#mum^2), AOD");
	hRECd0xd01D->Draw("hist");
	cd0xd0->cd();
	cd0xd0->SetLeftMargin(0.15);
	cd0xd0->SetRightMargin(0.05);
	hRECd0xd01D->Draw("hist");
	hRECd0xd01D->Draw("err same");
	cd0xd0->Update();

	cmc4->cd(6);
	TH1D *hRECAccd0xd01D = data->ShowProjection(id0xd0, stepRecAcc);
	maxd0xd0 = hRECAccd0xd01D->GetMaximum();
	hRECAccd0xd01D->GetYaxis()->SetRangeUser(0,maxd0xd0*1.2);
	SetHistoProperties(hRECAccd0xd01D,38,3,38,20,38,3007);
	hRECAccd0xd01D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccd0xd01D->Sumw2();
	hRECAccd0xd01D->SetMinimum(0.01);
	hRECAccd0xd01D->GetXaxis()->SetTitle("d0_{#pi}xd0_{K} (#mum^2), AOD - Acc cuts");
	hRECAccd0xd01D->Draw("hist");
	cAccd0xd0->cd();
	cAccd0xd0->SetLeftMargin(0.15);
	cAccd0xd0->SetRightMargin(0.05);
	hRECAccd0xd01D->Draw("hist");
	hRECAccd0xd01D->Draw("err same");
	cAccd0xd0->Update();

	cmc4->cd(7);
	TH1D *hRECITSCld0xd01D = data->ShowProjection(id0xd0, stepRecITSClus);
	maxd0xd0 = hRECITSCld0xd01D->GetMaximum();
	hRECITSCld0xd01D->GetYaxis()->SetRangeUser(0,maxd0xd0*1.2);
	SetHistoProperties(hRECITSCld0xd01D,13,3,13,20,13,3019);
	hRECITSCld0xd01D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSCld0xd01D->Sumw2();
	hRECITSCld0xd01D->SetMinimum(0.01);
	hRECITSCld0xd01D->GetXaxis()->SetTitle("d0_{#pi}xd0_{K} (#mum^2), AOD - ITS Cl cuts");
	hRECITSCld0xd01D->Draw("hist");
	cITSCld0xd0->cd();
	cITSCld0xd0->SetLeftMargin(0.15);
	cITSCld0xd0->SetRightMargin(0.05);
	hRECITSCld0xd01D->Draw("hist");
	hRECITSCld0xd01D->Draw("err same");
	cITSCld0xd0->Update();

	cmc4->cd(8);
	TH1D *hRECPPRd0xd01D = data->ShowProjection(id0xd0, stepRecPPR);
	maxd0xd0 = hRECPPRd0xd01D->GetMaximum();
	hRECPPRd0xd01D->GetYaxis()->SetRangeUser(0,maxd0xd0*1.2);
	SetHistoProperties(hRECPPRd0xd01D,6,3,6,20,6,3019);
	hRECPPRd0xd01D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRd0xd01D->Sumw2();
	hRECPPRd0xd01D->SetMinimum(0.01);
	hRECPPRd0xd01D->GetXaxis()->SetTitle("d0_{#pi}xd0_{K} (#mum^2), AOD - PPR cuts");
	hRECPPRd0xd01D->Draw("hist");
	cPPRd0xd0->cd();
	cPPRd0xd0->SetLeftMargin(0.15);
	cPPRd0xd0->SetRightMargin(0.05);
	hRECPPRd0xd01D->Draw("hist");
	hRECPPRd0xd01D->Draw("err same");
	cPPRd0xd0->Update();

	cmc4->cd(9);
	TH1D *hMCpointing1D = data->ShowProjection(ipointing, stepGen);
	Double_t maxpointing = hMCpointing1D->GetMaximum();
	hMCpointing1D->GetYaxis()->SetRangeUser(0,maxpointing*1.2);
	hMCpointing1D->Sumw2();
	hMCpointing1D->SetMinimum(0.01);
	SetHistoProperties(hMCpointing1D,2,3,2,20,2,3005);
	hMCpointing1D->GetXaxis()->SetTitleOffset(1.2);
	hMCpointing1D->GetXaxis()->SetTitle("cosPointingAngle, MC data");
	hMCpointing1D->Draw("hist");
	cmcpointing->cd();
	cmcpointing->SetLeftMargin(0.15);
	cmcpointing->SetRightMargin(0.05);
	hMCpointing1D->Draw("hist");
	hMCpointing1D->Draw("err same");
	cmcpointing->Update();

	cmc4->cd(10);
	TH1D *hMCAccpointing1D = data->ShowProjection(ipointing, stepGenAcc);
	maxpointing = hMCAccpointing1D->GetMaximum();
	hMCAccpointing1D->GetYaxis()->SetRangeUser(0,maxpointing*1.2);
	SetHistoProperties(hMCAccpointing1D,8,3,8,20,8,3006);
	hMCAccpointing1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAccpointing1D->Sumw2();
	hMCAccpointing1D->SetMinimum(0.01);
	hMCAccpointing1D->GetXaxis()->SetTitle("cosPointingAngle, MC - Acc cut");
	hMCAccpointing1D->Draw("hist");
	cmcAccpointing->cd();
	cmcAccpointing->SetLeftMargin(0.15);
	cmcAccpointing->SetRightMargin(0.05);
	hMCAccpointing1D->Draw("hist");
	hMCAccpointing1D->Draw("err same");
	cmcAccpointing->Update();

	cmc4->cd(11);
	TH1D *hMCVertexpointing1D = data->ShowProjection(ipointing, stepGenVertex);
	maxpointing = hMCVertexpointing1D->GetMaximum();
	hMCVertexpointing1D->GetYaxis()->SetRangeUser(0,maxpointing*1.2);
	SetHistoProperties(hMCVertexpointing1D,49,3,49,20,49,3006);
	hMCVertexpointing1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexpointing1D->Sumw2();
	hMCVertexpointing1D->SetMinimum(0.01);
	hMCVertexpointing1D->GetXaxis()->SetTitle("cosPointingAngle, MC - Vertex cut");
	hMCVertexpointing1D->Draw("hist");
	cmcVertexpointing->cd();
	cmcVertexpointing->SetLeftMargin(0.15);
	cmcVertexpointing->SetRightMargin(0.05);
	hMCVertexpointing1D->Draw("hist");
	hMCVertexpointing1D->Draw("err same");
	cmcVertexpointing->Update();

	cmc4->cd(12);
	TH1D *hMCRefitpointing1D = data->ShowProjection(ipointing, stepGenRefit);
	maxpointing = hMCRefitpointing1D->GetMaximum();
	hMCRefitpointing1D->GetYaxis()->SetRangeUser(0,maxpointing*1.2);
	SetHistoProperties(hMCRefitpointing1D,42,3,42,20,42,3006);
	hMCRefitpointing1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitpointing1D->Sumw2();
	hMCRefitpointing1D->SetMinimum(0.01);
	hMCRefitpointing1D->GetXaxis()->SetTitle("cosPointingAngle, MC - Refit cut");
	hMCRefitpointing1D->Draw("hist");
	cmcRefitpointing->cd();
	cmcRefitpointing->SetLeftMargin(0.15);
	cmcRefitpointing->SetRightMargin(0.05);
	hMCRefitpointing1D->Draw("hist");
	hMCRefitpointing1D->Draw("err same");
	cmcRefitpointing->Update();

	cmc4->cd(13);
	TH1D *hRECpointing1D = data->ShowProjection(ipointing, stepRec);
	maxpointing = hRECpointing1D->GetMaximum();
	hRECpointing1D->GetYaxis()->SetRangeUser(0,maxpointing*1.2);
	SetHistoProperties(hRECpointing1D,4,3,4,20,4,3004);
	hRECpointing1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECpointing1D->Sumw2();
	hRECpointing1D->SetMinimum(0.01);
	hRECpointing1D->GetXaxis()->SetTitle("cosPointingAngle, AOD");
	hRECpointing1D->Draw("hist");
	cpointing->cd();
	cpointing->SetLeftMargin(0.15);
	cpointing->SetRightMargin(0.05);
	hRECpointing1D->Draw("hist");
	hRECpointing1D->Draw("err same");
	cpointing->Update();

	cmc4->cd(14);
	TH1D *hRECAccpointing1D = data->ShowProjection(ipointing, stepRecAcc);
	maxpointing = hRECAccpointing1D->GetMaximum();
	hRECAccpointing1D->GetYaxis()->SetRangeUser(0,maxpointing*1.2);
	SetHistoProperties(hRECAccpointing1D,38,3,38,20,38,3007);
	hRECAccpointing1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccpointing1D->Sumw2();
	hRECAccpointing1D->SetMinimum(0.01);
	hRECAccpointing1D->GetXaxis()->SetTitle("cosPointingAngle, AOD - Acc cuts");
	hRECAccpointing1D->Draw("hist");
	cAccpointing->cd();
	cAccpointing->SetLeftMargin(0.15);
	cAccpointing->SetRightMargin(0.05);
	hRECAccpointing1D->Draw("hist");
	hRECAccpointing1D->Draw("err same");
	cAccpointing->Update();

	cmc4->cd(15);
	TH1D *hRECITSClpointing1D = data->ShowProjection(ipointing, stepRecITSClus);
	maxpointing = hRECITSClpointing1D->GetMaximum();
	hRECITSClpointing1D->GetYaxis()->SetRangeUser(0,maxpointing*1.2);
	SetHistoProperties(hRECITSClpointing1D,13,3,13,20,13,3019);
	hRECITSClpointing1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSClpointing1D->Sumw2();
	hRECITSClpointing1D->SetMinimum(0.01);
	hRECITSClpointing1D->GetXaxis()->SetTitle("cosPointingAngle, AOD - ITS Cl cuts");
	hRECITSClpointing1D->Draw("hist");
	cITSClpointing->cd();
	cITSClpointing->SetLeftMargin(0.15);
	cITSClpointing->SetRightMargin(0.05);
	hRECITSClpointing1D->Draw("hist");
	hRECITSClpointing1D->Draw("err same");
	cITSClpointing->Update();

	cmc4->cd(16);
	TH1D *hRECPPRpointing1D = data->ShowProjection(ipointing, stepRecPPR);
	maxpointing = hRECPPRpointing1D->GetMaximum();
	hRECPPRpointing1D->GetYaxis()->SetRangeUser(0,maxpointing*1.2);
	SetHistoProperties(hRECPPRpointing1D,6,3,6,20,6,3019);
	hRECPPRpointing1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRpointing1D->Sumw2();
	hRECPPRpointing1D->SetMinimum(0.01);
	hRECPPRpointing1D->GetXaxis()->SetTitle("cosPointingAngle, AOD - PPR cuts");
	hRECPPRpointing1D->Draw("hist");
	cPPRpointing->cd();
	cPPRpointing->SetLeftMargin(0.15);
	cPPRpointing->SetRightMargin(0.05);
	hRECPPRpointing1D->Draw("hist");
	hRECPPRpointing1D->Draw("err same");
	cPPRpointing->Update();

	cmc4->cd(17);
	TH1D *hMCphi1D = data->ShowProjection(iphi, stepGen);
	Double_t maxphi = hMCphi1D->GetMaximum();
	hMCphi1D->GetYaxis()->SetRangeUser(0,maxphi*1.2);
	hMCphi1D->Sumw2();
	hMCphi1D->SetMinimum(0.01);
	SetHistoProperties(hMCphi1D,2,3,2,20,2,3005);
	hMCphi1D->GetXaxis()->SetTitleOffset(1.2);
	hMCphi1D->GetXaxis()->SetTitle("Phi, MC data");
	hMCphi1D->Draw("hist");
	cmcphi->cd();
	cmcphi->SetLeftMargin(0.15);
	cmcphi->SetRightMargin(0.05);
	hMCphi1D->Draw("hist");
	hMCphi1D->Draw("err same");
	cmcphi->Update();

	cmc4->cd(18);
	TH1D *hMCAccphi1D = data->ShowProjection(iphi, stepGenAcc);
	maxphi = hMCAccphi1D->GetMaximum();
	hMCAccphi1D->GetYaxis()->SetRangeUser(0,maxphi*1.2);
	SetHistoProperties(hMCAccphi1D,8,3,8,20,8,3006);
	hMCAccphi1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCAccphi1D->Sumw2();
	hMCAccphi1D->SetMinimum(0.01);
	hMCAccphi1D->GetXaxis()->SetTitle("Phi, MC - Acc cut");
	hMCAccphi1D->Draw("hist");
	cmcAccphi->cd();
	cmcAccphi->SetLeftMargin(0.15);
	cmcAccphi->SetRightMargin(0.05);
	hMCAccphi1D->Draw("hist");
	hMCAccphi1D->Draw("err same");
	cmcAccphi->Update();

	cmc4->cd(19);
	TH1D *hMCVertexphi1D = data->ShowProjection(iphi, stepGenVertex);
	maxphi = hMCVertexphi1D->GetMaximum();
	hMCVertexphi1D->GetYaxis()->SetRangeUser(0,maxphi*1.2);
	SetHistoProperties(hMCVertexphi1D,49,3,49,20,49,3006);
	hMCVertexphi1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCVertexphi1D->Sumw2();
	hMCVertexphi1D->SetMinimum(0.01);
	hMCVertexphi1D->GetXaxis()->SetTitle("Phi, MC - Vertex cut");
	hMCVertexphi1D->Draw("hist");
	cmcVertexphi->cd();
	cmcVertexphi->SetLeftMargin(0.15);
	cmcVertexphi->SetRightMargin(0.05);
	hMCVertexphi1D->Draw("hist");
	hMCVertexphi1D->Draw("err same");
	cmcVertexphi->Update();

	cmc4->cd(20);
	TH1D *hMCRefitphi1D = data->ShowProjection(iphi, stepGenRefit);
	maxphi = hMCRefitphi1D->GetMaximum();
	hMCRefitphi1D->GetYaxis()->SetRangeUser(0,maxphi*1.2);
	SetHistoProperties(hMCRefitphi1D,42,3,42,20,42,3006);
	hMCRefitphi1D->GetXaxis()->SetTitleOffset(1.2);;
	hMCRefitphi1D->Sumw2();
	hMCRefitphi1D->SetMinimum(0.01);
	hMCRefitphi1D->GetXaxis()->SetTitle("Phi, MC - Refit cut");
	hMCRefitphi1D->Draw("hist");
	cmcRefitphi->cd();
	cmcRefitphi->SetLeftMargin(0.15);
	cmcRefitphi->SetRightMargin(0.05);
	hMCRefitphi1D->Draw("hist");
	hMCRefitphi1D->Draw("err same");
	cmcRefitphi->Update();

	cmc4->cd(21);
	TH1D *hRECphi1D = data->ShowProjection(iphi, stepRec);
	maxphi = hRECphi1D->GetMaximum();
	hRECphi1D->GetYaxis()->SetRangeUser(0,maxphi*1.2);
	SetHistoProperties(hRECphi1D,4,3,4,20,4,3004);
	hRECphi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECphi1D->Sumw2();
	hRECphi1D->SetMinimum(0.01);
	hRECphi1D->GetXaxis()->SetTitle("Phi, AOD");
	hRECphi1D->Draw("hist");
	cphi->cd();
	cphi->SetLeftMargin(0.15);
	cphi->SetRightMargin(0.05);
	hRECphi1D->Draw("hist");
	hRECphi1D->Draw("err same");
	cphi->Update();

	cmc4->cd(22);
	TH1D *hRECAccphi1D = data->ShowProjection(iphi, stepRecAcc);
	maxphi = hRECAccphi1D->GetMaximum();
	hRECAccphi1D->GetYaxis()->SetRangeUser(0,maxphi*1.2);
	SetHistoProperties(hRECAccphi1D,38,3,38,20,38,3007);
	hRECAccphi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECAccphi1D->Sumw2();
	hRECAccphi1D->SetMinimum(0.01);
	hRECAccphi1D->GetXaxis()->SetTitle("Phi, AOD - Acc cuts");
	hRECAccphi1D->Draw("hist");
	cAccphi->cd();
	cAccphi->SetLeftMargin(0.15);
	cAccphi->SetRightMargin(0.05);
	hRECAccphi1D->Draw("hist");
	hRECAccphi1D->Draw("err same");
	cAccphi->Update();

	cmc4->cd(23);
	TH1D *hRECITSClphi1D = data->ShowProjection(iphi, stepRecITSClus);
	maxphi = hRECITSClphi1D->GetMaximum();
	hRECITSClphi1D->GetYaxis()->SetRangeUser(0,maxphi*1.2);
	SetHistoProperties(hRECITSClphi1D,13,3,13,20,13,3019);
	hRECITSClphi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECITSClphi1D->Sumw2();
	hRECITSClphi1D->SetMinimum(0.01);
	hRECITSClphi1D->GetXaxis()->SetTitle("Phi, AOD - ITS Cl cuts");
	hRECITSClphi1D->Draw("hist");
	cITSClphi->cd();
	cITSClphi->SetLeftMargin(0.15);
	cITSClphi->SetRightMargin(0.05);
	hRECITSClphi1D->Draw("hist");
	hRECITSClphi1D->Draw("err same");
	cITSClphi->Update();

	cmc4->cd(24);
	TH1D *hRECPPRphi1D = data->ShowProjection(iphi, stepRecPPR);
	maxphi = hRECPPRphi1D->GetMaximum();
	hRECPPRphi1D->GetYaxis()->SetRangeUser(0,maxphi*1.2);
	SetHistoProperties(hRECPPRphi1D,6,3,6,20,6,3019);
	hRECPPRphi1D->GetXaxis()->SetTitleOffset(1.2);;
	hRECPPRphi1D->Sumw2();
	hRECPPRphi1D->SetMinimum(0.01);
	hRECPPRphi1D->GetXaxis()->SetTitle("Phi, AOD - PPR cuts");
	hRECPPRphi1D->Draw("hist");
	cPPRphi->cd();
	cPPRphi->SetLeftMargin(0.15);
	cPPRphi->SetRightMargin(0.05);
	hRECPPRphi1D->Draw("hist");
	hRECPPRphi1D->Draw("err same");
	cPPRphi->Update();

	// printing on eps files
	cmc1->Print("Plots/dataMC_pt_y_cTS.eps");
	cmc2->Print("Plots/dataMC_ptPi_ptK_cT.eps");
	cmc3->Print("Plots/dataMC_dca_d0pi_d0K.eps");
	cmc4->Print("Plots/dataMC_d0xd0_pointing.eps");
	cmcpt->Print("Plots/pt_Gen.eps");
	cmcy->Print("Plots/y_Gen.eps");
	cmccTS->Print("Plots/cTS_Gen.eps");
	cmcptPi->Print("Plots/ptPi_Gen.eps");
	cmcptK->Print("Plots/ptK_Gen.eps");
	cmccT->Print("Plots/cT_Gen.eps");
	cmcdca->Print("Plots/dca_Gen.eps");
	cmcd0pi->Print("Plots/d0pi_Gen.eps");
	cmcd0K->Print("Plots/d0K_Gen.eps");
	cmcd0xd0->Print("Plots/d0xd0_Gen.eps");
	cmcpointing->Print("Plots/pointing_Gen.eps");
	cmcphi->Print("Plots/phi_Gen.eps");

	cmcAccpt->Print("Plots/pt_Gen_Acc.eps");
	cmcAccy->Print("Plots/y_Gen_Acc.eps");
	cmcAcccTS->Print("Plots/cTS_Gen_Acc.eps");
	cmcAccptPi->Print("Plots/ptPi_Gen_Acc.eps");
	cmcAccptK->Print("Plots/ptK_Gen_Acc.eps");
	cmcAcccT->Print("Plots/cT_Gen_Acc.eps");
	cmcAccdca->Print("Plots/dca_Gen_Acc.eps");
	cmcAccd0pi->Print("Plots/d0pi_Gen_Acc.eps");
	cmcAccd0K->Print("Plots/d0K_Gen_Acc.eps");
	cmcAccd0xd0->Print("Plots/d0xd0_Gen_Acc.eps");
	cmcAccpointing->Print("Plots/pointing_Gen_Acc.eps");
	cmcAccphi->Print("Plots/phi_Gen_Acc.eps");

	cmcVertexpt->Print("Plots/pt_Gen_Vertex.eps");
	cmcVertexy->Print("Plots/y_Gen_Vertex.eps");
	cmcVertexcTS->Print("Plots/cTS_Gen_Vertex.eps");
	cmcVertexptPi->Print("Plots/ptPi_Gen_Vertex.eps");
	cmcVertexptK->Print("Plots/ptK_Gen_Vertex.eps");
	cmcVertexcT->Print("Plots/cT_Gen_Vertex.eps");
	cmcVertexdca->Print("Plots/dca_Gen_Vertex.eps");
	cmcVertexd0pi->Print("Plots/d0pi_Gen_Vertex.eps");
	cmcVertexd0K->Print("Plots/d0K_Gen_Vertex.eps");
	cmcVertexd0xd0->Print("Plots/d0xd0_Gen_Vertex.eps");
	cmcVertexpointing->Print("Plots/pointing_Gen_Vertex.eps");
	cmcVertexphi->Print("Plots/phi_Gen_Vertex.eps");

	cmcRefitpt->Print("Plots/pt_Gen_Refit.eps");
	cmcRefity->Print("Plots/y_Gen_Refit.eps");
	cmcRefitcTS->Print("Plots/cTS_Gen_Refit.eps");
	cmcRefitptPi->Print("Plots/ptPi_Gen_Refit.eps");
	cmcRefitptK->Print("Plots/ptK_Gen_Refit.eps");
	cmcRefitcT->Print("Plots/cT_Gen_Refit.eps");
	cmcRefitdca->Print("Plots/dca_Gen_Refit.eps");
	cmcRefitd0pi->Print("Plots/d0pi_Gen_Refit.eps");
	cmcRefitd0K->Print("Plots/d0K_Gen_Refit.eps");
	cmcRefitd0xd0->Print("Plots/d0xd0_Gen_Refit.eps");
	cmcRefitpointing->Print("Plots/pointing_Gen_Refit.eps");
	cmcRefitphi->Print("Plots/phi_Gen_Refit.eps");

	cpt->Print("Plots/pt_Rec.eps");
	cy->Print("Plots/y_Rec.eps");
	ccTS->Print("Plots/cTS_Rec.eps");
	cptPi->Print("Plots/ptPi_Rec.eps");
	cptK->Print("Plots/ptK_Rec.eps");
	ccT->Print("Plots/cT_Rec.eps");
	cdca->Print("Plots/dca_Rec.eps");
	cd0pi->Print("Plots/d0pi_Rec.eps");
	cd0K->Print("Plots/d0K_Rec.eps");
	cd0xd0->Print("Plots/d0xd0_Rec.eps");
	cpointing->Print("Plots/pointing_Rec.eps");
	cphi->Print("Plots/phi_Rec.eps");

	cAccpt->Print("Plots/pt_Rec_Acc.eps");
	cAccy->Print("Plots/y_Rec_Acc.eps");
	cAcccTS->Print("Plots/cTS_Rec_Acc.eps");
	cAccptPi->Print("Plots/ptPi_Rec_Acc.eps");
	cAccptK->Print("Plots/ptK_Rec_Acc.eps");
	cAcccT->Print("Plots/cT_Rec_Acc.eps");
	cmcdca->Print("Plots/dca_Rec_Acc.eps");
	cAccd0pi->Print("Plots/d0pi_Rec_Acc.eps");
	cAccd0K->Print("Plots/d0K_Rec_Acc.eps");
	cAccd0xd0->Print("Plots/d0xd0_Rec_Acc.eps");
	cAccpointing->Print("Plots/pointing_Rec_Acc.eps");
	cAccphi->Print("Plots/phi_Rec_Acc.eps");

	cITSClpt->Print("Plots/pt_Rec_ITSCl.eps");
	cITSCly->Print("Plots/y_Rec_ITSCl.eps");
	cITSClcTS->Print("Plots/cTS_Rec_ITSCl.eps");
	cITSClptPi->Print("Plots/ptPi_Rec_ITSCl.eps");
	cITSClptK->Print("Plots/ptK_Rec_ITSCl.eps");
	cITSClcT->Print("Plots/cT_Rec_ITSCl.eps");
	cITSCldca->Print("Plots/dca_Rec_ITSCl.eps");
	cITSCld0pi->Print("Plots/d0pi_Rec_ITSCl.eps");
	cITSCld0K->Print("Plots/d0K_Rec_ITSCl.eps");
	cITSCld0xd0->Print("Plots/d0xd0_Rec_ITSCl.eps");
	cITSClpointing->Print("Plots/pointing_Rec_ITSCl.eps");
	cITSClphi->Print("Plots/phi_Rec_ITSCl.eps");

	cPPRpt->Print("Plots/pt_Rec_PPR.eps");
	cPPRy->Print("Plots/y_Rec_PPR.eps");
	cPPRcTS->Print("Plots/cTS_Rec_PPR.eps");
	cPPRptPi->Print("Plots/ptPi_Rec_PPR.eps");
	cPPRptK->Print("Plots/ptK_Rec_PPR.eps");
	cPPRcT->Print("Plots/cT_Rec_PPR.eps");
	cPPRdca->Print("Plots/dca_Rec_PPR.eps");
	cPPRd0pi->Print("Plots/d0pi_Rec_PPR.eps");
	cPPRd0K->Print("Plots/d0K_Rec_PPR.eps");
	cPPRd0xd0->Print("Plots/d0xd0_Rec_PPR.eps");
	cPPRpointing->Print("Plots/pointing_Rec_PPR.eps");
	cPPRphi->Print("Plots/phi_Rec_PPR.eps");

	// printing on gif files
	cmc1->Print("Plots/dataMC_pt_y_cTS.gif");
	cmc2->Print("Plots/dataMC_ptPi_ptK_cT.gif");
	cmc3->Print("Plots/dataMC_dca_d0pi_d0K.gif");
	cmc4->Print("Plots/dataMC_d0xd0_pointing.gif");
	cmcpt->Print("Plots/pt_Gen.gif");
	cmcy->Print("Plots/y_Gen.gif");
	cmccTS->Print("Plots/cTS_Gen.gif");
	cmcptPi->Print("Plots/ptPi_Gen.gif");
	cmcptK->Print("Plots/ptK_Gen.gif");
	cmccT->Print("Plots/cT_Gen.gif");
	cmcdca->Print("Plots/dca_Gen.gif");
	cmcd0pi->Print("Plots/d0pi_Gen.gif");
	cmcd0K->Print("Plots/d0K_Gen.gif");
	cmcd0xd0->Print("Plots/d0xd0_Gen.gif");
	cmcpointing->Print("Plots/pointing_Gen.gif");
	cmcphi->Print("Plots/phi_Gen.gif");

	cmcAccpt->Print("Plots/pt_Gen_Acc.gif");
	cmcAccy->Print("Plots/y_Gen_Acc.gif");
	cmcAcccTS->Print("Plots/cTS_Gen_Acc.gif");
	cmcAccptPi->Print("Plots/ptPi_Gen_Acc.gif");
	cmcAccptK->Print("Plots/ptK_Gen_Acc.gif");
	cmcAcccT->Print("Plots/cT_Gen_Acc.gif");
	cmcAccdca->Print("Plots/dca_Gen_Acc.gif");
	cmcAccd0pi->Print("Plots/d0pi_Gen_Acc.gif");
	cmcAccd0K->Print("Plots/d0K_Gen_Acc.gif");
	cmcAccd0xd0->Print("Plots/d0xd0_Gen_Acc.gif");
	cmcAccpointing->Print("Plots/pointing_Gen_Acc.gif");
	cmcAccphi->Print("Plots/phi_Gen_Acc.gif");

	cmcVertexpt->Print("Plots/pt_Gen_Vertex.gif");
	cmcVertexy->Print("Plots/y_Gen_Vertex.gif");
	cmcVertexcTS->Print("Plots/cTS_Gen_Vertex.gif");
	cmcVertexptPi->Print("Plots/ptPi_Gen_Vertex.gif");
	cmcVertexptK->Print("Plots/ptK_Gen_Vertex.gif");
	cmcVertexcT->Print("Plots/cT_Gen_Vertex.gif");
	cmcVertexdca->Print("Plots/dca_Gen_Vertex.gif");
	cmcVertexd0pi->Print("Plots/d0pi_Gen_Vertex.gif");
	cmcVertexd0K->Print("Plots/d0K_Gen_Vertex.gif");
	cmcVertexd0xd0->Print("Plots/d0xd0_Gen_Vertex.gif");
	cmcVertexpointing->Print("Plots/pointing_Gen_Vertex.gif");
	cmcVertexphi->Print("Plots/phi_Gen_Vertex.gif");

	cmcRefitpt->Print("Plots/pt_Gen_Refit.gif");
	cmcRefity->Print("Plots/y_Gen_Refit.gif");
	cmcRefitcTS->Print("Plots/cTS_Gen_Refit.gif");
	cmcRefitptPi->Print("Plots/ptPi_Gen_Refit.gif");
	cmcRefitptK->Print("Plots/ptK_Gen_Refit.gif");
	cmcRefitcT->Print("Plots/cT_Gen_Refit.gif");
	cmcRefitdca->Print("Plots/dca_Gen_Refit.gif");
	cmcRefitd0pi->Print("Plots/d0pi_Gen_Refit.gif");
	cmcRefitd0K->Print("Plots/d0K_Gen_Refit.gif");
	cmcRefitd0xd0->Print("Plots/d0xd0_Gen_Refit.gif");
	cmcRefitpointing->Print("Plots/pointing_Gen_Refit.gif");
	cmcRefitphi->Print("Plots/phi_Gen_Refit.gif");

	cpt->Print("Plots/pt_Rec.gif");
	cy->Print("Plots/y_Rec.gif");
	ccTS->Print("Plots/cTS_Rec.gif");
	cptPi->Print("Plots/ptPi_Rec.gif");
	cptK->Print("Plots/ptK_Rec.gif");
	ccT->Print("Plots/cT_Rec.gif");
	cdca->Print("Plots/dca_Rec.gif");
	cd0pi->Print("Plots/d0pi_Rec.gif");
	cd0K->Print("Plots/d0K_Rec.gif");
	cd0xd0->Print("Plots/d0xd0_Rec.gif");
	cpointing->Print("Plots/pointing_Rec.gif");
	cphi->Print("Plots/phi_Rec.gif");

	cAccpt->Print("Plots/pt_Rec_Acc.gif");
	cAccy->Print("Plots/y_Rec_Acc.gif");
	cAcccTS->Print("Plots/cTS_Rec_Acc.gif");
	cAccptPi->Print("Plots/ptPi_Rec_Acc.gif");
	cAccptK->Print("Plots/ptK_Rec_Acc.gif");
	cAcccT->Print("Plots/cT_Rec_Acc.gif");
	cmcdca->Print("Plots/dca_Rec_Acc.gif");
	cAccd0pi->Print("Plots/d0pi_Rec_Acc.gif");
	cAccd0K->Print("Plots/d0K_Rec_Acc.gif");
	cAccd0xd0->Print("Plots/d0xd0_Rec_Acc.gif");
	cAccpointing->Print("Plots/pointing_Rec_Acc.gif");
	cAccphi->Print("Plots/phi_Rec_Acc.gif");

	cITSClpt->Print("Plots/pt_Rec_ITSCl.gif");
	cITSCly->Print("Plots/y_Rec_ITSCl.gif");
	cITSClcTS->Print("Plots/cTS_Rec_ITSCl.gif");
	cITSClptPi->Print("Plots/ptPi_Rec_ITSCl.gif");
	cITSClptK->Print("Plots/ptK_Rec_ITSCl.gif");
	cITSClcT->Print("Plots/cT_Rec_ITSCl.gif");
	cITSCldca->Print("Plots/dca_Rec_ITSCl.gif");
	cITSCld0pi->Print("Plots/d0pi_Rec_ITSCl.gif");
	cITSCld0K->Print("Plots/d0K_Rec_ITSCl.gif");
	cITSCld0xd0->Print("Plots/d0xd0_Rec_ITSCl.gif");
	cITSClpointing->Print("Plots/pointing_Rec_ITSCl.gif");
	cITSClphi->Print("Plots/phi_Rec_ITSCl.gif");

	cPPRpt->Print("Plots/pt_Rec_PPR.gif");
	cPPRy->Print("Plots/y_Rec_PPR.gif");
	cPPRcTS->Print("Plots/cTS_Rec_PPR.gif");
	cPPRptPi->Print("Plots/ptPi_Rec_PPR.gif");
	cPPRptK->Print("Plots/ptK_Rec_PPR.gif");
	cPPRcT->Print("Plots/cT_Rec_PPR.gif");
	cPPRdca->Print("Plots/dca_Rec_PPR.gif");
	cPPRd0pi->Print("Plots/d0pi_Rec_PPR.gif");
	cPPRd0K->Print("Plots/d0K_Rec_PPR.gif");
	cPPRd0xd0->Print("Plots/d0xd0_Rec_PPR.gif");
	cPPRpointing->Print("Plots/pointing_Rec_PPR.gif");
	cPPRphi->Print("Plots/phi_Rec_PPR.gif");

	TFile* file_histo = new TFile("fileHisto.root","RECREATE");
	// MC
	hMCpt1D->Write("hMCpt1D");
	hMCy1D->Write("hMCy1D");
	hMCcTS1D->Write("hMCcTS1D");
	hMCptPi1D->Write("hMCptPi1D");
	hMCptK1D->Write("hMCptK1D");
	hMCcT1D->Write("hMCcT1D");
	hMCdca1D->Write("hMCdca1D");
	hMCd0pi1D->Write("hMCd0pi1D");
	hMCd0K1D->Write("hMCd0K1D");
	hMCd0xd01D->Write("hMCd0xd01D");
	hMCpointing1D->Write("hMCpointing1D");
	hMCphi1D->Write("hMCphi1D");
	// MC, Acc
	hMCAccpt1D->Write("hMCAccpt1D");
	hMCAccy1D->Write("hMCAccy1D");
	hMCAcccTS1D->Write("hMCAcccTS1D");
	hMCAccptPi1D->Write("hMCAccptPi1D");
	hMCAccptK1D->Write("hMCAccptK1D");
	hMCAcccT1D->Write("hMCAcccT1D");
	hMCAccdca1D->Write("hMCAccdca1D");
	hMCAccd0pi1D->Write("hMCAccd0pi1D");
	hMCAccd0K1D->Write("hMCAccd0K1D");
	hMCAccd0xd01D->Write("hMCAccd0xd01D");
	hMCAccpointing1D->Write("hMCAccpointing1D");
	hMCAccphi1D->Write("hMCAccphi1D");
	// REC
	hRECpt1D->Write("hRECpt1D");
	hRECy1D->Write("hRECy1D");
	hRECcTS1D->Write("hRECcTS1D");
	hRECptPi1D->Write("hRECptPi1D");
	hRECptK1D->Write("hRECptK1D");
	hRECcT1D->Write("hRECcT1D");
	hRECdca1D->Write("hRECdca1D");
	hRECd0pi1D->Write("hRECd0pi1D");
	hRECd0K1D->Write("hRECd0K1D");
	hRECd0xd01D->Write("hRECd0xd01D");
	hRECpointing1D->Write("hRECpointing1D");
	hRECphi1D->Write("hRECphi1D");
	// REC, Acc
	hRECAccpt1D->Write("hRECAccpt1D");
	hRECAccy1D->Write("hRECAccy1D");
	hRECAcccTS1D->Write("hRECAcccTS1D");
	hRECAccptPi1D->Write("hRECAccptPi1D");
	hRECAccptK1D->Write("hRECAccptK1D");
	hRECAcccT1D->Write("hRECAcccT1D");
	hRECAccdca1D->Write("hRECAccdca1D");
	hRECAccd0pi1D->Write("hRECAccd0pi1D");
	hRECAccd0K1D->Write("hRECAccd0K1D");
	hRECAccd0xd01D->Write("hRECAccd0xd01D");
	hRECAccpointing1D->Write("hRECAccpointing1D");
	hRECAccphi1D->Write("hRECAccphi1D");
	// REC, ITS Cl
	hRECITSClpt1D->Write("hRECITSClpt1D");
	hRECITSCly1D->Write("hRECITSCly1D");
	hRECITSClcTS1D->Write("hRECITSClcTS1D");
	hRECITSClptPi1D->Write("hRECITSClptPi1D");
	hRECITSClptK1D->Write("hRECITSClptK1D");
	hRECITSClcT1D->Write("hRECITSClcT1D");
	hRECITSCldca1D->Write("hRECITSCldca1D");
	hRECITSCld0pi1D->Write("hRECITSCld0pi1D");
	hRECITSCld0K1D->Write("hRECITSCld0K1D");
	hRECITSCld0xd01D->Write("hRECITSCld0xd01D");
	hRECITSClpointing1D->Write("hRECITSClpointing1D");
	hRECITSClphi1D->Write("hRECITSClphi1D");
	// REC, PPR
	hRECPPRpt1D->Write("hRECPPRpt1D");
	hRECPPRy1D->Write("hRECPPRy1D");
	hRECPPRcTS1D->Write("hRECPPRcTS1D");
	hRECPPRptPi1D->Write("hRECPPRptPi1D");
	hRECPPRptK1D->Write("hRECPPRptK1D");
	hRECPPRcT1D->Write("hRECPPRcT1D");
	hRECPPRdca1D->Write("hRECPPRdca1D");
	hRECPPRd0pi1D->Write("hRECPPRd0pi1D");
	hRECPPRd0K1D->Write("hRECPPRd0K1D");
	hRECPPRd0xd01D->Write("hRECPPRd0xd01D");
	hRECPPRpointing1D->Write("hRECPPRpointing1D");
	hRECPPRphi1D->Write("hRECPPRphi1D");

}

void SetHistoProperties(TH1D* h, Int_t lineColor, Int_t lineWidth, Int_t markerColor, Int_t markerStyle, Int_t fillColor, Int_t fillStyle){

	h->SetLineColor(lineColor);
	h->SetLineWidth(lineWidth);
	h->SetMarkerColor(markerColor);
	h->SetFillColor(fillColor);
	h->SetFillStyle(fillStyle);
	h->SetMarkerStyle(markerStyle);

	return;

}
 
